package com.senter.demo.temperature.activity;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.SharedPreferences;
import android.graphics.Color;
import android.os.Bundle;
import android.os.SystemClock;
import android.preference.PreferenceManager;
import android.text.InputFilter;
import android.text.InputType;
import android.util.Log;
import android.view.KeyEvent;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.WindowManager;
import android.view.inputmethod.EditorInfo;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;
import android.widget.Toast;

import com.github.mikephil.charting.charts.LineChart;
import com.github.mikephil.charting.components.Description;
import com.github.mikephil.charting.components.Legend;
import com.github.mikephil.charting.components.XAxis;
import com.github.mikephil.charting.components.YAxis;
import com.github.mikephil.charting.data.Entry;
import com.github.mikephil.charting.data.LineData;
import com.github.mikephil.charting.data.LineDataSet;
import com.github.mikephil.charting.formatter.IValueFormatter;
import com.github.mikephil.charting.interfaces.datasets.ILineDataSet;
import com.github.mikephil.charting.utils.ColorTemplate;
import com.github.mikephil.charting.utils.ViewPortHandler;
import com.senter.demo.temperature.R;
import com.senter.demo.temperature.until.EmissivityEditInputFilter;
import com.senter.demo.temperature.until.ScreenListener;
import com.senter.function.openapi.unstable.InfraredThermometer;

public class TemperatureActivity extends Activity implements View.OnClickListener{
    private static final String TAG="TemperatureActivity";

    private Button buttonOpen;
    private TextView showtemperature;
    private EditText show_power;
    private LineChart mlinechart;
    
    private final InfraredThermometer thermometer = InfraredThermometer.getInstance();
    
    private final ScreenListener screenListener = new ScreenListener(this);

    private SharedPreferences sharedPreferences;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);

        initView();
        
        screenListener.begin(new ScreenListener.ScreenStateListener() {

            @Override
            public void onUserPresent() {
            }

            @Override
            public void onScreenOn() {
            }

            @Override
            public void onScreenOff() {
                thermometerStop();
            }
        });
    }
    private final InfraredThermometer.TemperatureListener temperatureListener=new InfraredThermometer.TemperatureListener() {
        @Override
        public void onNewTemperature(InfraredThermometer.ObjectTemperature temperature) {
            final Float eParameter =temperature.getAsObjectTemperatureByCentigrade();
            Log.v(TAG,"temperatureListener:"+eParameter);
            final String str=String.format("%.1f",eParameter);

            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    addEntry(eParameter);
                    showtemperature.setText(str + "℃");
                }
            });
        }
    };

    private void initView() {
        mlinechart = (LineChart) findViewById(R.id.linechart);
        Description description = new Description();
        description.setText(this.getString(R.string.time)+"  ");
        mlinechart.getDescription().setEnabled(true);
        mlinechart.setDragEnabled(true);
        mlinechart.setScaleEnabled(true);
        mlinechart.setDescription(description);
        mlinechart.setTouchEnabled(true);
        mlinechart.setDrawGridBackground(false);
        // if disabled, scaling can be done on x- and y-axis separately
        mlinechart.setPinchZoom(true);
        LineData data = new LineData();
        data.setValueTextColor(Color.WHITE);
        // add empty data
        mlinechart.setData(data);

        // get the legend (only possible after setting data)
        Legend l = mlinechart.getLegend();

        // modify the legend ...
        l.setForm(Legend.LegendForm.LINE);
        l.setTextColor(Color.WHITE);

        XAxis xl = mlinechart.getXAxis();
        xl.setTextColor(Color.GRAY);
        xl.setDrawGridLines(false);
        xl.setGranularity(0.5f);
        xl.setAvoidFirstLastClipping(true);
        xl.setEnabled(true);

        YAxis leftAxis = mlinechart.getAxisLeft();
        leftAxis.setTextColor(Color.GRAY);
        leftAxis.setAxisMinimum(0f);
        leftAxis.setSpaceTop(20f);
        leftAxis.setDrawGridLines(false);


        YAxis rightAxis = mlinechart.getAxisRight();
        rightAxis.setEnabled(false);

        showtemperature = (TextView) findViewById(R.id.showtemperature);
        show_power = (EditText) findViewById(R.id.show_power);

        String name = sharedPreferences.getString("saveMssage", "");
        if (!(name == null || name.equals(""))) {
            show_power.setText(name);
        }
        buttonOpen = (Button) findViewById(R.id.open);

        XAxis mXAxis = mlinechart.getXAxis();
        mXAxis.setPosition(XAxis.XAxisPosition.BOTTOM);
        buttonOpen.setOnClickListener(this);
        show_power.setFocusable(false);
        InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
        imm.toggleSoftInput(0, InputMethodManager.HIDE_NOT_ALWAYS);

        show_power.setOnEditorActionListener(new TextView.OnEditorActionListener() {
            public boolean onEditorAction(TextView v, int actionId, KeyEvent event) {
                if (actionId == EditorInfo.IME_ACTION_SEND || (event != null && event.getKeyCode() == KeyEvent.KEYCODE_ENTER)) {
                    show_power.setText(show_power.getText().toString());
                    show_power.setFocusable(false);
                    InputMethodManager imm = (InputMethodManager) getSystemService(Context.INPUT_METHOD_SERVICE);
                    imm.toggleSoftInput(0, InputMethodManager.HIDE_NOT_ALWAYS);
                    return true;
                }
                return false;
            }
        });
    }


    @Override
    public void onClick(View view) {
        switch (view.getId()) {
    
            case R.id.open: {
                if (thermometer.isWorking()) {
                    thermometerStop();
                } else {
                    thermometer.setCurrentObjectEmissivity(Float.valueOf(show_power.getText().toString()));
                    long pre = SystemClock.elapsedRealtime();
                    boolean started = thermometer.start(temperatureListener);
                    Log.e(TAG, "start:" + (SystemClock.elapsedRealtime() - pre));
                    if (started == true) {
                        getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
                        buttonOpen.setText(this.getString(R.string.close));
                        Toast.makeText(getApplication(), this.getString(R.string.show_message), Toast.LENGTH_SHORT).show();
                    } else {
                        buttonOpen.setText(this.getString(R.string.open));
                        Toast.makeText(getApplication(), "初始化失败", Toast.LENGTH_SHORT).show();
                    }
                }
                break;
            }
        }
    }
    
    private void thermometerStop(){
        if (thermometer.isWorking()){
            thermometer.stop();
            buttonOpen.setText(this.getString(R.string.open));
            getWindow().clearFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);
        }
    }


    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        getMenuInflater().inflate(R.menu.activity_main, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case R.id.menu_settings:
                showAddDialog();
                break;
            default:
                break;
        }
        return super.onOptionsItemSelected(item);
    }


    public boolean onKeyDown(int keyCode, KeyEvent event) {
        if (keyCode == KeyEvent.KEYCODE_BACK) {
            thermometerStop();
        }
        return super.onKeyDown(keyCode, event);
    }
    
    @Override
    protected void onDestroy() {
        screenListener.unregisterListener();
        thermometerStop();
        super.onDestroy();
    }
    
    /**
     * Add data
     */
    private void addEntry(float message) {
        Log.v(TAG,"addEntry:"+message);
        LineData data = mlinechart.getData();

        if (data != null) {
            ILineDataSet set = data.getDataSetByIndex(0);

            if (set == null) {
                set = createSet();
                data.addDataSet(set);
            }

            data.addEntry(new Entry(set.getEntryCount(), message), 0);
            data.notifyDataChanged();

            mlinechart.notifyDataSetChanged();
            mlinechart.setVisibleXRangeMaximum((int) 10);
            mlinechart.moveViewToX(data.getEntryCount());
        }
    }

    private LineDataSet createSet() {

        LineDataSet set = new LineDataSet(null, "");
        set.setAxisDependency(YAxis.AxisDependency.LEFT);
        set.setColor(ColorTemplate.getHoloBlue());
        set.setCircleColor(Color.GRAY);
        set.setDrawCircles(true);
        set.setLineWidth(2f);
        set.setCircleRadius(2f);
        set.setFillAlpha(25);
        
        set.setFillColor(ColorTemplate.getHoloBlue());

        // Sets the format of the data displayed on the broken line. If not set, the default display float data format.
        set.setValueFormatter(new IValueFormatter() {
            @Override
            public String getFormattedValue(float value, Entry entry, int dataSetIndex, ViewPortHandler viewPortHandler) {
                String str = "";
                return str;
            }
        });
        return set;
    }

    /**
     * Pop up dialog
     */
    public void showAddDialog() {

        LayoutInflater factory = LayoutInflater.from(TemperatureActivity.this);
        final View textEntryView = factory.inflate(R.layout.dialogview, null);
        final EditText editTextName = (EditText) textEntryView.findViewById(R.id.editTextName);
        InputFilter[] filters = {new EmissivityEditInputFilter()};
        editTextName.setFilters(filters);
        editTextName.setInputType(InputType.TYPE_CLASS_NUMBER | InputType.TYPE_NUMBER_FLAG_DECIMAL);
        AlertDialog.Builder ad1 = new AlertDialog.Builder(TemperatureActivity.this);
        ad1.setTitle(this.getString(R.string.revise));
        ad1.setView(textEntryView);
        ad1.setPositiveButton(this.getString(R.string.save) , new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int i) {
                String parameter = editTextName.getText().toString();
                Float number=  Float.parseFloat(parameter);
                if(number==0){
                    Toast.makeText(getApplication(), getResources().getString(R.string.zero_message), Toast.LENGTH_SHORT).show();
                    showAddDialog();
                    return;
                }

                show_power.setText(parameter);
                thermometer.setCurrentObjectEmissivity(Float.parseFloat(parameter));

                sharedPreferences.edit().putString("saveMssage", parameter).commit();

                dialog.dismiss();
            }
        });
        ad1.setNegativeButton(this.getString(R.string.no), new DialogInterface.OnClickListener() {
            public void onClick(DialogInterface dialog, int i) {
                dialog.dismiss();
            }
        });
        ad1.show();
    }
}
