package com.senter.support.demo.printA;

import java.io.IOException;
import java.io.InputStream;

import com.senter.demo.common.misc.ActivityHelper;
import com.senter.function.openapi.unstable.PrinterA;
import com.senter.function.openapi.unstable.PrinterA.BarcodeTitlePosition;
import com.senter.function.openapi.unstable.PrinterA.BitImage;
import com.senter.function.openapi.unstable.PrinterA.BitImagePrintMode;

import android.app.Activity;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.Bundle;
import android.os.SystemClock;
import android.view.View;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.CompoundButton.OnCheckedChangeListener;
import android.widget.EditText;
import android.widget.RadioGroup;
import android.widget.TextView;


public class MainActivity extends Activity {
    ActivityHelper<MainActivity> at = new ActivityHelper<MainActivity>(this);

    private Views views;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        views = new Views();

        views.setUiAsOpened(false);
    }

    PrinterA printerA = PrinterA.getInstance();

    @Override
    protected void onStop() {
        if (printerA.isInitialized()) {
            onUiBtn_Uninit();
        }
        super.onStop();
    }

    protected void onUiBtn_Init() {
        Boolean b = printerA.init();
        if (b == null || b == false) {
            at.showToastShort(R.string.InitializeFailed);
            return;
        }

        views.setUiAsOpened(true);
    }

    protected void onUiBtn_Uninit() {
        printerA.uninit();

        views.setUiAsOpened(false);
    }

    protected void onUiBtn_PrintCharacters() {
        final String v = views.etPrintCharacters.getText().toString();
        if (v.length() == 0) {
            return;
        }
        printerA.printCharacters(v);
    }

    protected void onUiBtn_PrintAndFeedPape() {
        Integer points = integerOfTextView(views.etPrintAndFeedPape);
        if (points == null) {
            at.showToastShort(R.string.InvalidValue);
            return;
        }

        boolean range = (1 <= points) && (points <= 255);
        if (range == false) {
            at.showToastShort(R.string.MustBeIn1_255);
            return;
        }
        printerA.feedPape(points);
    }

    private static final Integer integerOfTextView(TextView textView) {
        try {
            String v = textView.getText().toString();
            Integer value = Integer.valueOf(v);
            return value;
        } catch (NumberFormatException e) {
            return null;
        }
    }


    protected void onUiBtn_SetLineSpacing() {

        Integer points = integerOfTextView(views.etSetLineSpacing);
        if (points == null) {
            at.showToastShort(R.string.InvalidValue);
            return;
        }

        boolean range = (0 <= points) && (points <= 255);
        if (range == false) {
            at.showToastShort(R.string.MustBeIn0_255);
            return;
        }

        printerA.setLineSpacing(points);

    }

    protected void onUiBtn_SetRightSideSpacingOfCharacter() {

        Integer points = integerOfTextView(views.etSetRightSideSpacingOfCharacter);
        if (points == null) {
            at.showToastShort(R.string.InvalidValue);
            return;
        }

        boolean range = (0 <= points) && (points <= 255);
        if (range == false) {
            at.showToastShort(R.string.MustBeIn0_255);
            return;
        }

        printerA.setRightSideSpacingOfCharacter(points);


    }

    protected void onUiBtn_SetRightMargin() {

        Integer points = integerOfTextView(views.etSetRightMargin);
        if (points == null) {
            at.showToastShort(R.string.InvalidValue);
            return;
        }

        boolean range = (0 <= points) && (points <= 255);
        if (range == false) {
            at.showToastShort(R.string.MustBeIn0_255);
            return;
        }

        printerA.setRightMargin(points);

    }

    protected void onUiBtn_SetLeftMargin() {

        Integer points = integerOfTextView(views.etSetLeftMargin);
        if (points == null) {
            at.showToastShort(R.string.InvalidValue);
            return;
        }

        boolean range = (0 <= points) && (points <= 255);
        if (range == false) {
            at.showToastShort(R.string.MustBeIn0_255);
            return;
        }

        printerA.setLeftMargin(points);

    }

    protected void onUiBtn_SetCharactorWidthScale() {

        Integer scale = integerOfTextView(views.etSetCharactorWidthScale);
        if (scale == null) {
            at.showToastShort(R.string.InvalidValue);
            return;
        }

        boolean range = (1 <= scale) && (scale <= 4);
        if (range == false) {
            at.showToastShort(R.string.MustBeIn1_4);
            return;
        }
        printerA.setCharactorWidthScale(scale);

    }

    protected void onUiBtn_SetCharactorHeightScale() {
        Integer scale = integerOfTextView(views.etSetCharactorHeightScale);
        if (scale == null) {
            at.showToastShort(R.string.InvalidValue);
            return;
        }

        boolean range = (1 <= scale) && (scale <= 8);
        if (range == false) {
            at.showToastShort(R.string.MustBeIn1_8);
            return;
        }

        printerA.setCharactorHeightScale(scale);
    }

    protected void onUiBtn_SetCharactorWidthAndHeightScale() {
        Integer scale = integerOfTextView(views.etSetCharactorWidthAndHeightScale);
        if (scale == null) {
            at.showToastShort(R.string.InvalidValue);
            return;
        }

        boolean range = (1 <= scale) && (scale <= 4);
        if (range == false) {
            at.showToastShort(R.string.MustBeIn1_4);
            return;
        }

        printerA.setCharactorWidthAndHeightScale(scale);
    }

    protected void onUiBtn_TurnUnderlineModeOnOrOff(boolean b) {
        printerA.turnUnderlineModeOnOrOff(b);


    }

    protected void onUiBtn_TurnOverlineModeOnOrOff(boolean b) {
        printerA.turnOverlineModeOnOrOff(b);
    }

    protected void onUiBtn_TurnReverseModeOnOrOff(boolean b) {
        printerA.turnReverseModeOnOrOff(b);
    }

    protected void onUiBtn_GetVersion() {
        String bb = printerA.getVersion();

        at.showToastShort("Version：" + bb);
    }

    protected void onUiBtn_IsPaperReady() {
        Boolean b = printerA.isPaperReady();

        at.showToastShort("Is paper ready：" + (b != null ? b : "unknown"));
    }

    protected void onUiBtn_PrintAndLineFeed() {
        printerA.printAndLineFeed();


    }

    protected void onUiBtn_SetBarcodeTitlePosition(BarcodeTitlePosition barcodeTitlePosition) {
        printerA.setBarcodeTitlePosition(barcodeTitlePosition);
    }

    protected void onUiBtn_SetBarcodeHeight() {
        Integer points = integerOfTextView(views.etSetBarcodeHeight);
        if (points == null) {
            at.showToastShort(R.string.InvalidValue);
            return;
        }

        boolean range = (1 <= points) && (points <= 255);
        if (range == false) {
            at.showToastShort(R.string.MustBeIn1_255);
            return;
        }
        printerA.setBarcodeHeight(Integer.valueOf(points));
    }

    protected void onUiBtn_PrintBarcode() {
        String barcode = views.etPrintBarcode.getText().toString();
        if (barcode == null || barcode.length() == 0) {
            at.showToastShort(R.string.PleaseInputBarcodeContent);
            return;
        }
        printerA.printBarcode(barcode);
    }

    private static boolean isBitImageWriten = false;

    protected void onUiBtn_PrintBitImage() {

        if (isBitImageWriten == false) {
            try {
                InputStream is = getResources().getAssets().open("StrawHat.bmp");
                Bitmap bitmap = BitmapFactory.decodeStream(is);
                is.close();

                BitImage bitImage = BitImage.newInstanceOf(bitmap);

                printerA.setBitImage(bitImage);
                SystemClock.sleep(500);
                isBitImageWriten = true;
            } catch (IOException e1) {
                e1.printStackTrace();
                at.showToastShort(R.string.CannotWriteSampleImage);
                return;
            }
        }

        printerA.printBitImage(BitImagePrintMode.Normal);
        views.setAsDefaultValues();
    }


    private final class Views {
        Button btnInit = (Button) findViewById(R.id.idMainActivity_Init_btn);
        Button btnUninit = (Button) findViewById(R.id.idMainActivity_Uninit_btn);

        Button btnPrintCharacters = (Button) findViewById(R.id.idMainActivity_PrintCharacters_btn);
        EditText etPrintCharacters = (EditText) findViewById(R.id.idMainActivity_PrintCharacters_et);

        Button btnPrintAndFeedPape = (Button) findViewById(R.id.idMainActivity_PrintAndFeedPape_btn);
        EditText etPrintAndFeedPape = (EditText) findViewById(R.id.idMainActivity_PrintAndFeedPape_et);

        Button btnSetLineSpacing = (Button) findViewById(R.id.idMainActivity_SetLineSpacing_btn);
        EditText etSetLineSpacing = (EditText) findViewById(R.id.idMainActivity_SetLineSpacing_et);

        Button btnSetRightSideSpacingOfCharacter = (Button) findViewById(R.id.idMainActivity_SetRightSideSpacingOfCharacter_btn);
        EditText etSetRightSideSpacingOfCharacter = (EditText) findViewById(R.id.idMainActivity_SetRightSideSpacingOfCharacter_et);

        Button btnSetRightMargin = (Button) findViewById(R.id.idMainActivity_SetRightMargin_btn);
        EditText etSetRightMargin = (EditText) findViewById(R.id.idMainActivity_SetRightMargin_et);

        Button btnSetLeftMargin = (Button) findViewById(R.id.idMainActivity_SetLeftMargin_btn);
        EditText etSetLeftMargin = (EditText) findViewById(R.id.idMainActivity_SetLeftMargin_et);

        Button btnSetCharactorWidthScale = (Button) findViewById(R.id.idMainActivity_SetCharactorWidthScale_btn);
        EditText etSetCharactorWidthScale = (EditText) findViewById(R.id.idMainActivity_SetCharactorWidthScale_et);

        Button btnSetCharactorHeightScale = (Button) findViewById(R.id.idMainActivity_SetCharactorHeightScale_btn);
        EditText etSetCharactorHeightScale = (EditText) findViewById(R.id.idMainActivity_SetCharactorHeightScale_et);

        Button btnSetCharactorWidthAndHeightScale = (Button) findViewById(R.id.idMainActivity_SetCharactorWidthAndHeightScale_btn);
        EditText etSetCharactorWidthAndHeightScale = (EditText) findViewById(R.id.idMainActivity_SetCharactorWidthAndHeightScale_et);

        CheckBox cbTurnUnderlineModeOnOrOff = (CheckBox) findViewById(R.id.idMainActivity_TurnUnderlineModeOnOrOff_cb);

        CheckBox cbTurnOverlineModeOnOrOff = (CheckBox) findViewById(R.id.idMainActivity_TurnOverlineModeOnOrOff_cb);

        CheckBox cbTurnReverseModeOnOrOff = (CheckBox) findViewById(R.id.idMainActivity_TurnReverseModeOnOrOff_cb);

        Button btnGetVersion = (Button) findViewById(R.id.idMainActivity_GetVersion_btn);

        Button btnIsPaperReady = (Button) findViewById(R.id.idMainActivity_IsPaperReady_btn);

        Button btnPrintAndLineFeed = (Button) findViewById(R.id.idMainActivity_PrintAndLineFeed_btn);

        RadioGroup rgSetBarcodeTitlePosition = (RadioGroup) findViewById(R.id.idMainActivity_SetBarcodeTitlePosition_rg);

        Button btnSetBarcodeHeight = (Button) findViewById(R.id.idMainActivity_SetBarcodeHeight_btn);
        EditText etSetBarcodeHeight = (EditText) findViewById(R.id.idMainActivity_SetBarcodeHeight_et);

        Button btnPrintBarcode = (Button) findViewById(R.id.idMainActivity_PrintBarcode_btn);
        EditText etPrintBarcode = (EditText) findViewById(R.id.idMainActivity_PrintBarcode_et);

        Button btnPrintBitImage = (Button) findViewById(R.id.strMainActivity_PrintBitImageSample_btn);

        public Views() {
            btnInit.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    onUiBtn_Init();
                }
            });

            btnUninit.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    onUiBtn_Uninit();
                }
            });

            btnPrintCharacters.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    onUiBtn_PrintCharacters();
                }
            });

            btnPrintAndFeedPape.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    onUiBtn_PrintAndFeedPape();
                }
            });

            btnSetLineSpacing.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    onUiBtn_SetLineSpacing();
                }
            });

            btnSetRightSideSpacingOfCharacter.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    onUiBtn_SetRightSideSpacingOfCharacter();
                }
            });

            btnSetRightMargin.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    onUiBtn_SetRightMargin();
                }
            });

            btnSetLeftMargin.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    onUiBtn_SetLeftMargin();
                }
            });

            btnSetCharactorWidthScale.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    onUiBtn_SetCharactorWidthScale();
                }
            });

            btnSetCharactorHeightScale.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    onUiBtn_SetCharactorHeightScale();
                }
            });

            btnSetCharactorWidthAndHeightScale.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    onUiBtn_SetCharactorWidthAndHeightScale();
                }
            });

            cbTurnUnderlineModeOnOrOff.setOnCheckedChangeListener(new OnCheckedChangeListener() {
                @Override
                public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                    onUiBtn_TurnUnderlineModeOnOrOff(isChecked);
                }
            });

            cbTurnOverlineModeOnOrOff.setOnCheckedChangeListener(new OnCheckedChangeListener() {
                @Override
                public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                    onUiBtn_TurnOverlineModeOnOrOff(isChecked);
                }
            });

            cbTurnReverseModeOnOrOff.setOnCheckedChangeListener(new OnCheckedChangeListener() {
                @Override
                public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
                    onUiBtn_TurnReverseModeOnOrOff(isChecked);
                }
            });

            btnGetVersion.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    onUiBtn_GetVersion();
                }
            });

            btnIsPaperReady.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    onUiBtn_IsPaperReady();
                }
            });

            btnPrintAndLineFeed.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    onUiBtn_PrintAndLineFeed();
                }
            });

            rgSetBarcodeTitlePosition.setOnCheckedChangeListener(new RadioGroup.OnCheckedChangeListener() {
                @Override
                public void onCheckedChanged(RadioGroup group, int checkedId) {
                    if (printerA.isInitialized() == false) {
                        return;
                    }
                    switch (checkedId) {
                        case R.id.idMainActivity_SetBarcodeTitlePosition_rb_Above: {
                            onUiBtn_SetBarcodeTitlePosition(BarcodeTitlePosition.AboveTheBarcode);
                            break;
                        }
                        case R.id.idMainActivity_SetBarcodeTitlePosition_rb_Below: {
                            onUiBtn_SetBarcodeTitlePosition(BarcodeTitlePosition.UnderTheBarcode);
                            break;
                        }
                        case R.id.idMainActivity_SetBarcodeTitlePosition_rb_Non: {
                            onUiBtn_SetBarcodeTitlePosition(BarcodeTitlePosition.Non);
                            break;
                        }
                        default:
                            break;
                    }
                }
            });

            btnSetBarcodeHeight.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    onUiBtn_SetBarcodeHeight();
                }
            });

            btnPrintBarcode.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    onUiBtn_PrintBarcode();
                }
            });

            btnPrintBitImage.setOnClickListener(new View.OnClickListener() {
                @Override
                public void onClick(View v) {
                    onUiBtn_PrintBitImage();
                }
            });
        }

        public void setUiAsOpened(boolean opened) {
            btnInit.setEnabled(!opened);
            btnUninit.setEnabled(opened);
            btnPrintCharacters.setEnabled(opened);
            btnPrintAndFeedPape.setEnabled(opened);
            btnSetLineSpacing.setEnabled(opened);
            btnSetRightSideSpacingOfCharacter.setEnabled(opened);
            btnSetRightMargin.setEnabled(opened);
            btnSetLeftMargin.setEnabled(opened);
            btnSetCharactorWidthScale.setEnabled(opened);
            btnSetCharactorHeightScale.setEnabled(opened);
            btnSetCharactorWidthAndHeightScale.setEnabled(opened);
            cbTurnUnderlineModeOnOrOff.setEnabled(opened);
            cbTurnOverlineModeOnOrOff.setEnabled(opened);
            cbTurnReverseModeOnOrOff.setEnabled(opened);
            btnGetVersion.setEnabled(opened);
            btnIsPaperReady.setEnabled(opened);
            btnPrintAndLineFeed.setEnabled(opened);
            rgSetBarcodeTitlePosition.setEnabled(opened);
            btnSetBarcodeHeight.setEnabled(opened);
            btnPrintBarcode.setEnabled(opened);
            btnPrintBitImage.setEnabled(opened);
            if (opened) {
                setAsDefaultValues();
            }
        }

        public void setAsDefaultValues() {
            etPrintAndFeedPape.setText("3");
            etPrintBarcode.setText("1234567890");
            etSetBarcodeHeight.setText("175");
            etSetCharactorHeightScale.setText("1");
            etSetCharactorWidthAndHeightScale.setText("1");
            etSetCharactorWidthScale.setText("1");
            etSetLeftMargin.setText("0");
            etSetLineSpacing.setText("0");
            etSetRightMargin.setText("0");
            etSetRightSideSpacingOfCharacter.setText("0");

            cbTurnOverlineModeOnOrOff.setChecked(false);
            cbTurnReverseModeOnOrOff.setChecked(false);
            cbTurnUnderlineModeOnOrOff.setChecked(false);

            rgSetBarcodeTitlePosition.check(R.id.idMainActivity_SetBarcodeTitlePosition_rb_Non);
        }
    }

}
