package com.senter.support.demo.rs485.activity;

import android.app.Activity;
import android.os.Bundle;
import android.text.Editable;
import android.text.Selection;
import android.text.Spannable;
import android.text.TextWatcher;
import android.util.Log;
import android.view.KeyEvent;
import android.view.View;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.CompoundButton;
import android.widget.CompoundButton.OnCheckedChangeListener;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;
import com.senter.function.openapi.unstable.Rs485;
import com.senter.function.openapi.unstable.Rs485.Rs485BitRate;
import com.senter.function.openapi.unstable.Rs485.Rs485CharSize;
import com.senter.function.openapi.unstable.Rs485.Rs485Parity;
import com.senter.function.openapi.unstable.Rs485.Rs485StopBits;
import com.senter.support.demo.rs485.R;
import com.senter.support.demo.rs485.until.FileUtil;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.regex.Pattern;

public class MainActivity extends Activity {
	private static final String	TAG				= "MainActivity";
	private long				mExitTime;

	private Rs485				rs485;

	private Views	views;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);
		views = new Views();// 初始化界面
	}
	@Override
	protected void onStop() {
		if (rs485!=null&&rs485.isInitialized()) {
			onUiBtn_Uninit();
		}
		super.onStop();
	}

	protected void onUiBtn_Init() {
		if(rs485!=null&&rs485.isInitialized())throw new IllegalStateException();
		
		Rs485BitRate rs485BitRate = views.rs485BitRate();// 波特率
		Rs485CharSize rs485CharSize = views.rs485CharSize();// 数据位
		Rs485Parity rs485Parity = views.rs485Parity();// 校验位
		Rs485StopBits rs485StopBits = views.rs485StopBits();// 停止位

		rs485 = Rs485.getNewInstance(rs485BitRate, rs485CharSize, rs485Parity, rs485StopBits);

		if (rs485.init() == false) {
			Toast.makeText(this, R.string.openFailure, Toast.LENGTH_SHORT).show();
			return;
		}
		
		ReadThread mReadThread = new ReadThread(); // 开启接受数据的线程
		mReadThread.start();

		views.setUiAsOpened(true);

		Toast.makeText(this, R.string.openSuccess, Toast.LENGTH_SHORT).show();
	}

	protected void onUiBtn_Uninit() {
		rs485.uninit();
		views.setUiAsOpened(false);

		Toast.makeText(this, R.string.closeSuccess, Toast.LENGTH_SHORT).show();
	}
	protected void onUiBtn_HexSendData(boolean enabled) {
		views.changeSendDataToHex(enabled);
	}
	protected void onUiBtn_HexReceivedData(boolean enabled) {
		views.changeReceivedDataToHex(enabled);
	}

	protected void onUiBtn_Send() {
		String message = views.sendDataEditext.getText().toString();
		if (message == null || message.equals("")) {
			Toast.makeText(this, R.string.empty_message, Toast.LENGTH_SHORT).show();
			return;
		}

		try {
			if (views.isHexDisplaySendData()) {
				byte[] data = FileUtil.toByteArray(message);
				rs485.write(data);
			} else {
				byte[] data = message.getBytes();
				rs485.write(data);
			}

		} catch (IOException e) {
			e.printStackTrace();
			throw new IllegalArgumentException(TAG + "发送数据异常");
		}
	}
	protected void onUiBtn_ClearReceivedData() {
		views.clearReceivedData();
	}
	protected void onUiBtn_ClearSendData() {
		views.clearSendData();
	}
	/**
	 * 返回键监听
	 */
	public boolean onKeyDown(int keyCode, KeyEvent event) {
		if (keyCode == KeyEvent.KEYCODE_BACK) {
			if ((System.currentTimeMillis() - mExitTime) > 2000) {
				Toast.makeText(this, R.string.sign_out, Toast.LENGTH_SHORT).show();
				mExitTime = System.currentTimeMillis();
			} else {
				finish();
			}
			return true;
		}
		return super.onKeyDown(keyCode, event);
	}

	/**
	 * 接受数据线程
	 */
	private class ReadThread extends Thread {
		@Override
		public void run() {
			super.run();

			Log.e(TAG, " 线程已经执行：  ");
			try {
				while (rs485.isInitialized()) {
					byte[] bytes = rs485.read();
					if (bytes == null) {
						return;
					}
					if (bytes.length == 0) {
						sleep(100);
						continue;
					}
					
					views.addReivedData(bytes);
				}
			} catch (IOException e) {
				e.printStackTrace();
				Log.e(TAG, " 读取线程接收到的终止信息，退出");
			} catch (InterruptedException e) {
				e.printStackTrace();
				Log.e(TAG, " 读取线程接收到的终止信息，退出");
			}catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	private class Views {

		private Spinner		bitRateSpinner					= (Spinner) findViewById(R.id.idMainActivity_Settings_spBitRate);// 波特率
		private Spinner		charSizeSpinner					= (Spinner) findViewById(R.id.idMainActivity_Settings_spCharSize);// 数据位
		private Spinner		paritySpinner					= (Spinner) findViewById(R.id.idMainActivity_Settings_spParity);// 校验位
		private Spinner		stopbitSpinner					= (Spinner) findViewById(R.id.idMainActivity_Settings_spStopBits);// 停止位

		private Button		openButton						= (Button) findViewById(R.id.idMainActivity_Init);			// 打开串口
		private Button		closeButton						= (Button) findViewById(R.id.idMainActivity_Uninit);		// 关闭串口
		
		private Button		sendButton						= (Button) findViewById(R.id.idMainActivity_Send_btnSend);	// 发送数据按钮
		private Button		sendDataClearButton						=(Button) findViewById(R.id.idMainActivity_Send_btnClear);
		private EditText	sendDataEditext						= (EditText) findViewById(R.id.idMainActivity_Send_etData);	// 发送的数据

		private TextView	receivedDataTextview					= (TextView) findViewById(R.id.idMainActivity_Receive_etData);// 接收区域
		private Button		receivedDataClearButton					=(Button) findViewById(R.id.idMainActivity_Receive_btnClear);
		
		private CheckBox	hexDisplayOfReceivedCheckbox	= (CheckBox) findViewById(R.id.idMainActivity_Receive_cbHex);// 接收checkbo选项
		private CheckBox	hexDisplayOfSendCheckbox		= (CheckBox) findViewById(R.id.idMainActivity_Send_cbHex);	// 发送checkbo选项

		public Views() {
			openButton.setOnClickListener(new View.OnClickListener() {
				@Override
				public void onClick(View v) {
					onUiBtn_Init();
				}
			});

			closeButton.setOnClickListener(new View.OnClickListener() {

				@Override
				public void onClick(View v) {
					onUiBtn_Uninit();
				}
			});

			sendButton.setOnClickListener(new View.OnClickListener() {
				@Override
				public void onClick(View v) {
					onUiBtn_Send();
				}
			});
			// 接收checkbox改变监听
			hexDisplayOfSendCheckbox.setOnCheckedChangeListener(new OnCheckedChangeListener() {
				@Override
				public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
					onUiBtn_HexSendData(isChecked);
				}
			});
			// 发送checkbox改变监听
			hexDisplayOfReceivedCheckbox.setOnCheckedChangeListener(new OnCheckedChangeListener() {
				@Override
				public void onCheckedChanged(CompoundButton buttonView, boolean isChecked) {
					onUiBtn_HexReceivedData(isChecked);
				}
			});

			receivedDataClearButton.setOnClickListener(new View.OnClickListener() {
				
				@Override
				public void onClick(View v) {
					onUiBtn_ClearReceivedData();
				}
			});
			
			sendDataClearButton.setOnClickListener(new View.OnClickListener() {
				
				@Override
				public void onClick(View v) {
					onUiBtn_ClearSendData();
				}
			});
		}

		public void setUiAsOpened(boolean opened) {

			if (opened == true) {
				views.openButton.setEnabled(false);
				bitRateSpinner.setEnabled(false);
				charSizeSpinner.setEnabled(false);
				paritySpinner.setEnabled(false);
				stopbitSpinner.setEnabled(false);
				
				views.closeButton.setEnabled(true);
				views.sendButton.setEnabled(true);
			} else {
				views.openButton.setEnabled(true);
				
				bitRateSpinner.setEnabled(true);
				charSizeSpinner.setEnabled(true);
				paritySpinner.setEnabled(true);
				stopbitSpinner.setEnabled(true);
				
				views.closeButton.setEnabled(false);
				views.sendButton.setEnabled(false);
			}
		}
		
		public void clearReceivedData(){
			receivedDataTextview.setText("");
		}
		public void clearSendData(){
			sendDataEditext.setText("");
		}

		public Rs485BitRate rs485BitRate() {

			String bpsMessage = views.bitRateSpinner.getSelectedItem().toString();
			Rs485BitRate rs485BitRate = FileUtil.Rs485BitRateOf(bpsMessage);// 波特率
			return rs485BitRate;
		}

		public Rs485CharSize rs485CharSize() {

			String dataMessage = views.charSizeSpinner.getSelectedItem().toString();
			Rs485CharSize rs485CharSize = FileUtil.Rs485CharSizeOf(dataMessage);// 数据位
			return rs485CharSize;
		}

		public Rs485Parity rs485Parity() {
			String parityMessage = views.paritySpinner.getSelectedItem().toString();
			Rs485Parity rs485Parity = FileUtil.Rs485ParityOf(parityMessage);// 校验位
			return rs485Parity;
		}

		public Rs485StopBits rs485StopBits() {
			String stopMessage = views.stopbitSpinner.getSelectedItem().toString();
			Rs485StopBits rs485StopBits = FileUtil.Rs485StopBitsOf(stopMessage);// 停止位
			return rs485StopBits;
		}

		public void addReivedData(final byte[] bytes){
			Log.e(TAG, " 接收到的数据信息为：  " + new String(bytes));
			runOnUiThread(new Runnable() {
				@Override
				public void run() {

					if (isHexDisplayOfReceivedData()) {
						String old=receivedDataTextview.getText().toString();
						String newData = FileUtil.toHexString(bytes);
						old=old+" "+newData;
						receivedDataTextview.setText(old);
					} else {
						String old=receivedDataTextview.getText().toString();
						String newData = old+new String(bytes);
						receivedDataTextview.setText(newData);
					}
				}
			});
		}
		
		public boolean isHexDisplaySendData(){
			return hexDisplayOfSendCheckbox.isChecked();
		}
		public boolean isHexDisplayOfReceivedData(){
			return hexDisplayOfReceivedCheckbox.isChecked();
		}
		final TextWatcher hexTextWatcher=new TextWatcher() {
			
			@Override
			public void onTextChanged(CharSequence s, int start, int before, int count) {
				
			}
			
			@Override
			public void beforeTextChanged(CharSequence s, int start, int count, int after) {
				
			}

			final Pattern pattern=Pattern.compile("[0-9a-fA-F]");
			@Override
			public void afterTextChanged(Editable s) {
				final String orgString=s.toString();
				
				String process=orgString.replaceAll("\\s", "");
				
				int i = 0;
				for (; i < process.length();i++) {
					if (pattern.matcher(process.subSequence(i, i+1)).matches()==false) {
						break;
					}
				}
				String pure16=process.substring(0,i);//至此，都是0-9a-fA-F
				StringBuffer sb=new StringBuffer();
				for (int j = 0; j < pure16.length(); j++) {
					if (j==0) {
						sb.append(pure16.charAt(j));
					}else if (j%2==1) {
						sb.append(pure16.charAt(j));
						sb.append(" ");
					}else {
						sb.append(pure16.charAt(j));
					}
				}
				String processed=sb.toString().trim();
				if (processed.equals(orgString)) {
					return;
				}else {
					s.replace(0, s.length(), processed);
					Log.e(TAG, "替换为：'"+processed+"'");
				}
			}
		};
		
		public void changeSendDataToHex(boolean isHex){
			String old=sendDataEditext.getText().toString();
			if (isHex) {//字符转十六进制
				String newData = FileUtil.toHexString(old.getBytes());
				sendDataEditext.setText(newData);
				sendDataEditext.addTextChangedListener(hexTextWatcher);
			}else {//十六进制转字符
				sendDataEditext.removeTextChangedListener(hexTextWatcher);
				try {
					String newData = new String(FileUtil.toByteArray(old));
					sendDataEditext.setText(newData);
				} catch (UnsupportedEncodingException e) {
					e.printStackTrace();
				}
			}
			
			CharSequence cs =sendDataEditext.getText();
			if (cs instanceof Spannable) {
				Selection.setSelection(Spannable.class.cast(cs), cs.length());
			}
		}

		public void changeReceivedDataToHex(boolean isHex){
			String old=receivedDataTextview.getText().toString();
			if (isHex) {//字符转十六进制
				String newData = FileUtil.toHexString(old.getBytes());
				receivedDataTextview.setText(newData);
			}else {//十六进制转字符
				try {
					String newData = new String(FileUtil.toByteArray(old));
					receivedDataTextview.setText(newData);
				} catch (UnsupportedEncodingException e) {
					e.printStackTrace();
				}
			}
		}
	}
}
