package com.senter.support.openapi.demo.barcodescanaidl;

import java.io.UnsupportedEncodingException;

import android.app.Activity;
import android.os.Bundle;
import android.view.View;
import android.widget.TextView;
import android.widget.Toast;

import com.senter.iot.support.openapi.StBarcodeScanOperator;
import com.senter.support.openapi.StBarcodeScanner;

/**
 * 当前类中写了sdk操作类的基本操作，首先通过isLibrarySupport()方法判断当前设备是否可以调用sdk相关方法。
 * 如果isLibrarySupport()返回false，请使用原条码库StBarcodeScanner类中的方法。
 */
public class MainActivity extends Activity {
    
    private TextView textview;
    
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        textview = (TextView) findViewById(R.id.textview);
        
        if (StBarcodeScanOperator.getInstance(this).isLibrarySupport()) {
            StBarcodeScanOperator.getInstance(this).init(new StBarcodeScanOperator.InitListener() {
                @Override
                public void onInit(StBarcodeScanOperator.ErrorCode code) {
                    //
                    if (code == StBarcodeScanOperator.ErrorCode.SUCCESS) {
                        //初始化成功
                        //TODO: do something
                        Toast.makeText(MainActivity.this, "初始化成功", Toast.LENGTH_SHORT).show();
                    } else {
                        Toast.makeText(MainActivity.this, "初始化失败", Toast.LENGTH_SHORT).show();
                    }
                }
            });
        } else {
            //不支持当前新版条码sdk
        }
        
        findViewById(R.id.button).setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                scan();
            }
        });
    }
    
    /**
     * 扫描
     */
    private void scan() {
        
        if (StBarcodeScanOperator.getInstance(this).isLibrarySupport()) {
            
            //支持新版sdk的调用，新版sdk调用如下
            
            if (!StBarcodeScanOperator.getInstance(MainActivity.this).isInited()) {
                return;
            }
            
            if (!StBarcodeScanOperator.getInstance(MainActivity.this).isEnable()) {
                Toast.makeText(MainActivity.this, "条码扫描未启用，前往条码扫描助手中开启", Toast.LENGTH_SHORT).show();
                return;
            }
            
            new Thread(new Runnable() {
                @Override
                public void run() {
                    try {
                        final StBarcodeScanOperator.BarcodeInfo barcodeInfo = StBarcodeScanOperator.getInstance(MainActivity.this).scan();
                        runOnUiThread(new Runnable() {
                            @Override
                            public void run() {
                                try {
                                    if (barcodeInfo != null && barcodeInfo.getBarcodeValueAsBytes() != null) {
                                        textview.setText("Utf8:" + new String(barcodeInfo.getBarcodeValueAsBytes(), "utf-8") + "\r\n" +
                                                                 "GBK:" + new String(barcodeInfo.getBarcodeValueAsBytes(), "GBK") + "\r\n");
                                    } else {
                                        textview.setText("无");
                                    }
                                } catch (UnsupportedEncodingException e) {
                                    e.printStackTrace();
                                }
                            }
                        });
                    } catch (StBarcodeScanOperator.BarcodeScanException e) {
                        e.printStackTrace();
                    }
                }
            }).start();
        } else {
            
            //不支持新版sdk的调用，旧版sdk处理如下
            
            new Thread(new Runnable() {
                @Override
                public void run() {
                    try {
                        final StBarcodeScanner.BarcodeInfo barcodeInfo = StBarcodeScanner.getInstance().scanBarcodeInfo();
                        runOnUiThread(new Runnable() {
                            @Override
                            public void run() {
                                try {
                                    if (barcodeInfo != null && barcodeInfo.getBarcodeValueAsBytes() != null) {
                                        textview.setText("Utf8:" + new String(barcodeInfo.getBarcodeValueAsBytes(), "utf-8") + "\r\n" +
                                                                 "GBK:" + new String(barcodeInfo.getBarcodeValueAsBytes(), "GBK") + "\r\n");
                                    } else {
                                        textview.setText("无");
                                    }
                                } catch (UnsupportedEncodingException e) {
                                    e.printStackTrace();
                                }
                            }
                        });
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    }
                    
                    
                }
            }).start();
            
        }
    }
    
    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (StBarcodeScanOperator.getInstance(this).isLibrarySupport()) {
            StBarcodeScanOperator.getInstance(this).uninit();
        }
    }
}
