package com.senter.support.openapi.demo.fingerprintb;

import java.io.File;
import java.io.IOException;
import com.senter.function.openapi.unstable.FingerprintB;
import com.senter.function.openapi.unstable.FingerprintB.ModuleMode1Api2;
import com.senter.function.openapi.unstable.FingerprintB.ModuleMode1Api2.ErrorCode;
import com.senter.function.openapi.unstable.FingerprintB.ModuleMode1Api2.FingerprintCharacterBuffer;
import com.senter.function.openapi.unstable.FingerprintB.ModuleMode1Api2.LibraryStorageInfo;
import com.senter.function.openapi.unstable.FingerprintB.ModuleMode1Api2.Password;
import com.senter.function.openapi.unstable.FingerprintB.ModuleMode1Api2.Result.ResultOfGetCharacter;
import com.senter.function.openapi.unstable.FingerprintB.ModuleMode1Api2.Result.ResultOfGetImage;
import com.senter.function.openapi.unstable.FingerprintB.ModuleMode1Api2.Result.ResultOfGetLibraryStorageInfo;
import com.senter.function.openapi.unstable.FingerprintB.ModuleMode1Api2.Result.ResultOfMatchCharacterBufferAAndB;
import com.senter.function.openapi.unstable.FingerprintB.ModuleMode1Api2.Result.ResultOfSearchFast;

import android.app.Activity;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.os.AsyncTask;
import android.os.Bundle;
import android.os.SystemClock;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

public class MainActivity extends Activity {
	private TextView				uiMessageTv;
	private ImageView				uiFingerprintIv;

	private final ModuleMode1Api2		moduleMode1		= FingerprintB.getInstance().getModuleAs(ModuleMode1Api2.class);
	private static final String		TAG				= "MainActivity";
	
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);

		uiMessageTv = (TextView) findViewById(R.id.textView2);
		uiFingerprintIv = (ImageView) findViewById(R.id.imageView1);

		onCreate_InitButtons();
		setUiAsOpenedOrClosed(false);
	}
	protected void onCreate_InitButtons(){

		final Button button = (Button) findViewById(R.id.idBtnOpenOrClose);

		button.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				onUiBtnOpenOrClose(button);
			}
		});

		Button btnMatch = (Button) findViewById(R.id.idBtnMatch);
		Button btnGetChar = (Button) findViewById(R.id.idBtnGetChar);
		Button btnGetImg = (Button) findViewById(R.id.idBtnGetImg);
		Button btnEnroll = (Button) findViewById(R.id.idBtnEnroll);
		Button btnSearch = (Button) findViewById(R.id.idBtnSearch);
		Button btnClear = (Button) findViewById(R.id.idBtnClear);

		btnMatch.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				onUiBtnMatch();
			}
		});

		btnGetChar.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				onUiBtnGetChar();
			}
		});

		btnGetImg.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				onUiBtnGetImg();
			}
		});

		btnEnroll.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				onUiBtnEnroll();
			}
		});

		btnSearch.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				onUiBtnSearch();
			}
		});

		btnClear.setOnClickListener(new View.OnClickListener() {
			@Override
			public void onClick(View v) {
				onUiBtnClear();
			}
		});
	}
	
	private ErrorCode verifyDefaultPasswords(){

		ErrorCode ec=null;
		
		for (int i = 0; i < 3; i++) {
			ec=moduleMode1.verifyPassword(Password.newInstanceOfDefaultPassword1());
			if (ec==ErrorCode.Ok) {
				break;
			}else if (ec==ErrorCode.InvalidPassword) {
				ec=moduleMode1.verifyPassword(Password.newInstanceOfDefaultPassword2());
				if (ec==ErrorCode.Ok) {
					break;
				}else {
					continue;
				}
			}else {
				continue;
			}
		}
		if (ec==null) {
			return null;
		}
		
		return ec;
	}
	
	protected void onUiBtnEnroll() {
		SystemClock.sleep(500);

		
		
		new AsyncTask<Void, Void, Void>(){
			
			@Override
			protected void onPreExecute() {
				setUiAsOperating(true);
			}
			
			@Override
			protected Void doInBackground(Void... params) {
				
				int idleIndex;
				ResultOfGetLibraryStorageInfo rlsi=moduleMode1.getLibraryStorageInfo();
				if (rlsi==null) {
					return null;
				}
				if (rlsi.errorCode()==ErrorCode.VerifyPasswordPlease) {
					verifyDefaultPasswords();
				}
				
				rlsi=moduleMode1.getLibraryStorageInfo();
				if (rlsi.errorCode()==ErrorCode.Ok) {
					LibraryStorageInfo lsi=rlsi.libraryStorageInfo();
					if (lsi==null) {
						return null;
					}
					int[] unused=lsi.indexesUnusedInLibrary();
					if (unused==null) {
						return null;
					}
					if (unused.length==0) {
						idleIndex=0;
					}else {
						idleIndex=unused[0];
					}
				}else {
					
					return null;
				}
				
				
				
				
				
				int fpcharbuf = 1;
				{
					long	ssart			= SystemClock.elapsedRealtime();
					long	ssend			= ssart+1000*10;
					for (;;) {
						if (SystemClock.elapsedRealtime()>ssend) {
							String temp = getString(R.string.FingerprintTimeout) + "\r\n";
							setText(uiMessageTv,temp);
							return null;
						}
						
						ErrorCode nRet = moduleMode1.tryCaptureFingerprintOnSensor();

						switch (nRet) {
							case Ok:
							{
								nRet = moduleMode1.generateCharacterByImageOnChip(fpcharbuf);
								if (nRet != ErrorCode.Ok) {
									setText(uiMessageTv,getString(R.string.FeatureIsTooBad__PleaseRetry));
									SystemClock.sleep(1000);
									continue;
								}
								break;
							}
							case NoFinger:
							{
								String temp = getString(R.string.WaitingForFinger__TimeLeft_Colon) + ((10000 - (SystemClock.elapsedRealtime() - ssart))) / 1000 + "s";
								setText(uiMessageTv,temp);
								SystemClock.sleep(500);
								continue;
							}
							case VerifyPasswordPlease:
							{
								ErrorCode ec=verifyDefaultPasswords();
								
								if (ec==null) {
									return null;
								}else if (ec==ErrorCode.Ok) {
									continue;
								}else{
									return null;
								}
							}
							default:
							{
								String temp = getString(R.string.WaitingForFinger__TimeLeft_Colon) + ((10000 - (SystemClock.elapsedRealtime() - ssart))) / 1000 + "s";
								setText(uiMessageTv,temp);
								SystemClock.sleep(50);
								continue;
							}
						}
						break;
					}
				}

				setText(uiMessageTv,getString(R.string.PleasePickUpYourFinger));
				for (long begain=SystemClock.elapsedRealtime(); (SystemClock.elapsedRealtime()-begain)<(1000*10);) {
					if (moduleMode1.tryCaptureFingerprintOnSensor()==ErrorCode.NoFinger) {
						break;
					}
					SystemClock.sleep(100);
				}

				setText(uiMessageTv,getString(R.string.PleasePressYourFingerAgain));
				SystemClock.sleep(1000*1);
				
				fpcharbuf ++;
				{
					long	ssart			= SystemClock.elapsedRealtime();
					long	ssend			= ssart+1000*10;
					
					for (;;) {
						if (SystemClock.elapsedRealtime()>ssend) {
							String temp = getString(R.string.FingerprintTimeout) + "\r\n";
							setText(uiMessageTv,temp);
							return null;
						}
						
						ErrorCode nRet = moduleMode1.tryCaptureFingerprintOnSensor();

						switch (nRet) {
							case Ok:
							{
								nRet = moduleMode1.generateCharacterByImageOnChip(fpcharbuf + 1);
								if (nRet != ErrorCode.Ok) {
									String temp = getString(R.string.FeatureIsTooBad__PleaseRetry);
									setText(uiMessageTv,temp);
									SystemClock.sleep(500);
									continue;
								}
								break;
							}
							case NoFinger:
							{
								String temp = getString(R.string.WaitingForFinger__TimeLeft_Colon) + ((10000 - (SystemClock.elapsedRealtime() - ssart))) / 1000 + "s";
								setText(uiMessageTv,temp);
								SystemClock.sleep(500);
								continue;
							}
							case GetImgErr:
							{
								String temp = getString(R.string.GettingImage);
								Log.d(TAG, temp + ": " + nRet);
								setText(uiMessageTv,temp);
								SystemClock.sleep(50);
								continue;
							}
							case VerifyPasswordPlease:
							{
								ErrorCode ec=null;
								
								for (int i = 0; i < 3; i++) {
									ec=moduleMode1.verifyPassword(Password.newInstanceOfDefaultPassword1());
									if (ec==ErrorCode.Ok) {
										break;
									}else if (ec==ErrorCode.InvalidPassword) {
										ec=moduleMode1.verifyPassword(Password.newInstanceOfDefaultPassword2());
										if (ec==ErrorCode.Ok) {
											break;
										}else {
											continue;
										}
									}else {
										continue;
									}
								}
								if (ec==null) {
									return null;
								}else if (ec==ErrorCode.Ok) {
									continue;
								}else{
									return null;
								}
								
							}
							default:
							{
								String temp = getString(R.string.WaitingForFinger__TimeLeft_Colon) + ((10000 - (SystemClock.elapsedRealtime() - ssart))) / 1000 + "s";
								setText(uiMessageTv,temp);
								SystemClock.sleep(50);
								continue;
							}
						}
						break;
					}
				}
				
				

				ErrorCode nRet = moduleMode1.mergeCharacterBufferAAndB();
				if (nRet != ErrorCode.Ok) {
					setText(uiMessageTv,getString(R.string.MergeFingerprintFailed));
					return null;
				} else {
					nRet=moduleMode1.saveCharacter(1, idleIndex);
					if (nRet == ErrorCode.Ok) {
						setText(uiMessageTv,getString(R.string.EnrollSuccessfullyID_Colon) + idleIndex);
						return null;
					} else {
						setText(uiMessageTv,getString(R.string.EnrollFailed));
						return null;
					}
				}
			}
			
			@Override
			protected void onPostExecute(Void result) {
				setUiAsOperating(false);
			}
			
			
			@Override
			protected void onProgressUpdate(Void... values) {

			}
			
		}.execute(null,null);
	}

	private static String charToHexString(byte[] val) {
		String temp = "";
		for (int i = 0; i < val.length; i++) {
			String hex = Integer.toHexString(0xff & val[i]);
			if (hex.length() == 1) {
				hex = '0' + hex;
			}
			temp += hex.toUpperCase();
		}
		return temp;
	}

	protected void onUiBtnOpenOrClose(final Button button) {
		if (button.getText().equals(getString(R.string.openDevice))) {

			ErrorCode status=moduleMode1.init(Password.newInstanceOfDefaultPassword1());
			if (status==ErrorCode.InvalidPassword) {
				status=moduleMode1.init(Password.newInstanceOfDefaultPassword2());
			}
			Log.e(TAG, " open status: " + status);
			if (status==ErrorCode.Ok) {
				String temp = getString(R.string.openSuccessfully);
				Toast.makeText(MainActivity.this, temp, Toast.LENGTH_SHORT).show();
				button.setText(getString(R.string.closeDevice));
				setText(uiMessageTv,temp);
				setUiAsOpenedOrClosed(true);
			} else {
				Toast.makeText(MainActivity.this, R.string.OpenFailed, Toast.LENGTH_SHORT).show();
				String temp = getString(R.string.OpenFailed);
				setText(uiMessageTv,temp);
			}
		} else {
			moduleMode1.uninit();
			button.setText(getString(R.string.openDevice));
			setUiAsOpenedOrClosed(false);
		}
	}

	protected void onUiBtnMatch() {
		SystemClock.sleep(500);
		
		
		new AsyncTask<Void, Void, Void>(){
			
			@Override
			protected void onPreExecute() {
				setUiAsOperating(true);
			}
			
			@Override
			protected Void doInBackground(Void... params) {
				
				int fpcharbuf = 1;
				{
					long	ssart			= SystemClock.elapsedRealtime();
					long	ssend			= ssart+1000*10;
					for (;;) {
						if (SystemClock.elapsedRealtime()>ssend) {
							String temp = getString(R.string.FingerprintTimeout) + "\r\n";
							setText(uiMessageTv,temp);
							return null;
						}
						
						ErrorCode nRet = moduleMode1.tryCaptureFingerprintOnSensor();

						switch (nRet) {
							case Ok:
							{
								nRet = moduleMode1.generateCharacterByImageOnChip(fpcharbuf);
								if (nRet != ErrorCode.Ok) {
									setText(uiMessageTv,getString(R.string.FeatureIsTooBad__PleaseRetry));
									SystemClock.sleep(1000);
									continue;
								}
								break;
							}
							case NoFinger:
							{
								String temp = getString(R.string.WaitingForFinger__TimeLeft_Colon) + ((10000 - (SystemClock.elapsedRealtime() - ssart))) / 1000 + "s";
								setText(uiMessageTv,temp);
								SystemClock.sleep(500);
								continue;
							}
							case VerifyPasswordPlease:
							{
								ErrorCode ec=null;
								
								for (int i = 0; i < 3; i++) {
									ec=moduleMode1.verifyPassword(Password.newInstanceOfDefaultPassword1());
									if (ec==ErrorCode.Ok) {
										break;
									}else if (ec==ErrorCode.InvalidPassword) {
										ec=moduleMode1.verifyPassword(Password.newInstanceOfDefaultPassword2());
										if (ec==ErrorCode.Ok) {
											break;
										}else {
											continue;
										}
									}else {
										continue;
									}
								}
								if (ec==null) {
									return null;
								}else if (ec==ErrorCode.Ok) {
									continue;
								}else{
									return null;
								}
								
							}
							default:
							{
								String temp = getString(R.string.WaitingForFinger__TimeLeft_Colon) + ((10000 - (SystemClock.elapsedRealtime() - ssart))) / 1000 + "s";
								setText(uiMessageTv,temp);
								SystemClock.sleep(50);
								continue;
							}
						}
						break;
					}
				}

				setText(uiMessageTv,getString(R.string.PleasePickUpYourFinger));
				for (long begain=SystemClock.elapsedRealtime(); (SystemClock.elapsedRealtime()-begain)<(1000*10);) {
					if (moduleMode1.tryCaptureFingerprintOnSensor()==ErrorCode.NoFinger) {
						break;
					}
					SystemClock.sleep(100);
				}

				setText(uiMessageTv,getString(R.string.PleasePressYourFingerForMath));
				SystemClock.sleep(1000*1);
				
				fpcharbuf ++;
				{
					long	ssart			= SystemClock.elapsedRealtime();
					long	ssend			= ssart+1000*10;
					
					for (;;) {
						if (SystemClock.elapsedRealtime()>ssend) {
							String temp = getString(R.string.FingerprintTimeout) + "\r\n";
							setText(uiMessageTv,temp);
							return null;
						}
						
						ErrorCode nRet = moduleMode1.tryCaptureFingerprintOnSensor();

						switch (nRet) {
							case Ok:
							{
								nRet = moduleMode1.generateCharacterByImageOnChip(fpcharbuf + 1);
								if (nRet != ErrorCode.Ok) {
									String temp = getString(R.string.FeatureIsTooBad__PleaseRetry);
									setText(uiMessageTv,temp);
									SystemClock.sleep(500);
									continue;
								}
								break;
							}
							case NoFinger:
							{
								String temp = getString(R.string.WaitingForFinger__TimeLeft_Colon) + ((10000 - (SystemClock.elapsedRealtime() - ssart))) / 1000 + "s";
								setText(uiMessageTv,temp);
								SystemClock.sleep(500);
								continue;
							}
							case GetImgErr:
							{
								String temp = getString(R.string.GettingImage);
								Log.d(TAG, temp + ": " + nRet);
								setText(uiMessageTv,temp);
								SystemClock.sleep(50);
								continue;
							}
							case VerifyPasswordPlease:
							{
								ErrorCode ec=null;
								
								for (int i = 0; i < 3; i++) {
									ec=moduleMode1.verifyPassword(Password.newInstanceOfDefaultPassword1());
									if (ec==ErrorCode.Ok) {
										break;
									}else if (ec==ErrorCode.InvalidPassword) {
										ec=moduleMode1.verifyPassword(Password.newInstanceOfDefaultPassword2());
										if (ec==ErrorCode.Ok) {
											break;
										}else {
											continue;
										}
									}else {
										continue;
									}
								}
								if (ec==null) {
									return null;
								}else if (ec==ErrorCode.Ok) {
									continue;
								}else{
									return null;
								}
								
							}
							default:
							{
								String temp = getString(R.string.WaitingForFinger__TimeLeft_Colon) + ((10000 - (SystemClock.elapsedRealtime() - ssart))) / 1000 + "s";
								setText(uiMessageTv,temp);
								SystemClock.sleep(50);
								continue;
							}
						}
						break;
					}
				}
				
				
				ResultOfMatchCharacterBufferAAndB rslt=moduleMode1.matchCharacterBufferAAndB();
				if (rslt.errorCode() == ErrorCode.Ok) {
					setText(uiMessageTv,getString(R.string.MatchSucessfully));
				} else {
					setText(uiMessageTv,getString(R.string.Dismatch));
				}
				return null;
			}
			
			@Override
			protected void onPostExecute(Void result) {
				setUiAsOperating(false);
			}
			
			
			@Override
			protected void onProgressUpdate(Void... values) {

			}
			
		}.execute(null,null);
	}

	protected void onUiBtnGetChar() {
		SystemClock.sleep(500);
		
		
		new AsyncTask<Void, Void, Void>(){
			
			@Override
			protected void onPreExecute() {
				setUiAsOperating(true);
			}
			
			@Override
			protected Void doInBackground(Void... params) {
				long	ssart			= SystemClock.elapsedRealtime();
				long	ssend			= ssart+1000*10;
				for (;;) {
					if (SystemClock.elapsedRealtime()>ssend) {
						String temp = getString(R.string.FingerprintTimeout) + "\r\n";
						setText(uiMessageTv,temp);
						return null;
					}
					
					ErrorCode nRet = moduleMode1.tryCaptureFingerprintOnSensor();

					switch (nRet) {
						case Ok:
						{
							nRet = moduleMode1.generateCharacterByImageOnChip(1);
							if (nRet != ErrorCode.Ok) {
								setText(uiMessageTv,getString(R.string.FeatureIsTooBad__PleaseRetry));
								SystemClock.sleep(1000);
								continue;
							}
							break;
						}
						case NoFinger:
						{
							String temp = getString(R.string.WaitingForFinger__TimeLeft_Colon) + ((10000 - (SystemClock.elapsedRealtime() - ssart))) / 1000 + "s";
							setText(uiMessageTv,temp);
							SystemClock.sleep(500);
							continue;
						}
						case VerifyPasswordPlease:
						{
							ErrorCode ec=null;
							
							for (int i = 0; i < 3; i++) {
								ec=moduleMode1.verifyPassword(Password.newInstanceOfDefaultPassword1());
								if (ec==ErrorCode.Ok) {
									break;
								}else if (ec==ErrorCode.InvalidPassword) {
									ec=moduleMode1.verifyPassword(Password.newInstanceOfDefaultPassword2());
									if (ec==ErrorCode.Ok) {
										break;
									}else {
										continue;
									}
								}else {
									continue;
								}
							}
							if (ec==null) {
								return null;
							}else if (ec==ErrorCode.Ok) {
								continue;
							}else{
								return null;
							}
							
						}
						default:
						{
							String temp = getString(R.string.WaitingForFinger__TimeLeft_Colon) + ((10000 - (SystemClock.elapsedRealtime() - ssart))) / 1000 + "s";
							setText(uiMessageTv,temp);
							SystemClock.sleep(50);
							continue;
						}
					}
					break;
				}

				ResultOfGetCharacter rslt=moduleMode1.getCharacter(FingerprintCharacterBuffer.CharacterBufferA);
				if (rslt.errorCode() == ErrorCode.Ok) {
					String temp = getString(R.string.FingerprintCharacter)+"\r\n";
					temp += charToHexString(rslt.fingerprintCharacter().toByteArray());
					setText(uiMessageTv,temp);
				}else {
					String temp = getString(R.string.FeatureIsTooBad__PleaseRetry)+"\r\n";
					setText(uiMessageTv,temp);
				}
				
				return null;
			}
			
			@Override
			protected void onPostExecute(Void result) {
				setUiAsOperating(false);
			}
		}.execute(null,null);
	}

	protected void onUiBtnGetImg() {
		SystemClock.sleep(500);
		
		
		new AsyncTask<Void, Void, Void>(){
			
			@Override
			protected void onPreExecute() {
				setUiAsOperating(true);
			}
			
			@Override
			protected Void doInBackground(Void... params) {
				long	ssart			= SystemClock.elapsedRealtime();
				long	ssend			= ssart+1000*10;
				for (;;) {
					if (SystemClock.elapsedRealtime()>ssend) {
						String temp = getString(R.string.FingerprintTimeout) + "\r\n";
						setText(uiMessageTv,temp);
						return null;
					}
					
					ErrorCode nRet = moduleMode1.tryCaptureFingerprintOnSensor();

					switch (nRet) {
						case Ok:
						{
							break;
						}
						case NoFinger:
						{
							String temp = getString(R.string.WaitingForFinger__TimeLeft_Colon) + ((10000 - (SystemClock.elapsedRealtime() - ssart))) / 1000 + "s";
							setText(uiMessageTv,temp);
							SystemClock.sleep(500);
							continue;
						}
						case VerifyPasswordPlease:
						{
							ErrorCode ec=null;
							
							for (int i = 0; i < 3; i++) {
								ec=moduleMode1.verifyPassword(Password.newInstanceOfDefaultPassword1());
								if (ec==ErrorCode.Ok) {
									break;
								}else if (ec==ErrorCode.InvalidPassword) {
									ec=moduleMode1.verifyPassword(Password.newInstanceOfDefaultPassword2());
									if (ec==ErrorCode.Ok) {
										break;
									}else {
										continue;
									}
								}else {
									continue;
								}
							}
							if (ec==null) {
								return null;
							}else if (ec==ErrorCode.Ok) {
								continue;
							}else{
								return null;
							}
							
						}
						default:
						{
							String temp = getString(R.string.WaitingForFinger__TimeLeft_Colon) + ((10000 - (SystemClock.elapsedRealtime() - ssart))) / 1000 + "s";
							setText(uiMessageTv,temp);
							SystemClock.sleep(50);
							continue;
						}
					}
					break;
				}
				
				setText(uiMessageTv,getString(R.string.FingerprintCollectionOk_YouCanPickUpYourFinger_Uploading));
				ResultOfGetImage img = null;
				for (;;) {
					img=moduleMode1.getImage();
					if (img!=null&&img.errorCode()==ErrorCode.Ok) {
						break;
					}
				}
				String str = "/mnt/sdcard/test.bmp";
				try {
					moduleMode1.saveImage(img.fingerprintImage(), new File(str));
					
					final Bitmap bmpDefaultPic = BitmapFactory.decodeFile(str, null);
					runOnUiThread(new Runnable() {
						@Override
						public void run() {
							uiFingerprintIv.setImageBitmap(bmpDefaultPic);
							uiMessageTv.setText(getString(R.string.GetImageSuccessfully)+" "+getString(R.string.Resolution_Colon)+" "+bmpDefaultPic.getWidth()+"x"+bmpDefaultPic.getHeight());
						}
					});
				} catch (IOException e) {
					e.printStackTrace();
					Toast.makeText(MainActivity.this, e.getMessage(), Toast.LENGTH_SHORT).show();
				}
				return null;
			}
			
			@Override
			protected void onPostExecute(Void result) {
				setUiAsOperating(false);
			}
		}.execute(null,null);
	}

	protected void onUiBtnSearch() {
		SystemClock.sleep(500);

		new AsyncTask<Void, Void, Void>(){
			
			@Override
			protected void onPreExecute() {
				setUiAsOperating(true);
			}
			
			@Override
			protected Void doInBackground(Void... params) {
				long	ssart			= SystemClock.elapsedRealtime();
				long	ssend			= ssart+1000*10;
				for (;;) {
					if (SystemClock.elapsedRealtime()>ssend) {
						String temp = getString(R.string.FingerprintTimeout) + "\r\n";
						setText(uiMessageTv,temp);
						return null;
					}
					
					ErrorCode nRet = moduleMode1.tryCaptureFingerprintOnSensor();

					switch (nRet) {
						case Ok:
						{
							nRet = moduleMode1.generateCharacterByImageOnChip(1);
							if (nRet != ErrorCode.Ok) {
								setText(uiMessageTv,getString(R.string.FeatureIsTooBad__PleaseRetry));
								SystemClock.sleep(1000);
								continue;
							}
							break;
						}
						case NoFinger:
						{
							String temp = getString(R.string.WaitingForFinger__TimeLeft_Colon) + ((10000 - (SystemClock.elapsedRealtime() - ssart))) / 1000 + "s";
							setText(uiMessageTv,temp);
							SystemClock.sleep(500);
							continue;
						}
						case VerifyPasswordPlease:
						{
							ErrorCode ec=null;
							
							for (int i = 0; i < 3; i++) {
								ec=moduleMode1.verifyPassword(Password.newInstanceOfDefaultPassword1());
								if (ec==ErrorCode.Ok) {
									break;
								}else if (ec==ErrorCode.InvalidPassword) {
									ec=moduleMode1.verifyPassword(Password.newInstanceOfDefaultPassword2());
									if (ec==ErrorCode.Ok) {
										break;
									}else {
										continue;
									}
								}else {
									continue;
								}
							}
							if (ec==null) {
								return null;
							}else if (ec==ErrorCode.Ok) {
								continue;
							}else{
								return null;
							}
							
						}
						default:
						{
							String temp = getString(R.string.WaitingForFinger__TimeLeft_Colon) + ((10000 - (SystemClock.elapsedRealtime() - ssart))) / 1000 + "s";
							setText(uiMessageTv,temp);
							SystemClock.sleep(50);
							continue;
						}
					}
					break;
				}
				
				ResultOfSearchFast id=moduleMode1.searchFast(1, 0, 1000);
				if (id!=null&&id.errorCode()!=null&&id.errorCode() == ErrorCode.Ok) {
					setText(uiMessageTv,getString(R.string.SearchFingerprintSuccessfully__ID_Colon) + id.indexOfTemplateInLibrary());
					setUiAsOperating(false);
				} else {
					setText(uiMessageTv,getString(R.string.SearchFingerprintFailed));
					setUiAsOperating(false);
				}
				
				return null;
			}
			
			@Override
			protected void onPostExecute(Void result) {
				setUiAsOperating(false);
			}
		}.execute(null,null);
	}

	protected void onUiBtnClear() {
		ErrorCode ec=moduleMode1.deleteAllTemplatesOnChip();
		if (ec==ErrorCode.VerifyPasswordPlease) {
			verifyDefaultPasswords();
		}
		ec=moduleMode1.deleteAllTemplatesOnChip();
		String temp = getString(R.string.ClearLibrarySuccessfully) + "\r\n";
		setText(uiMessageTv,temp);
	}
	
	/**
	 * If some function is operating,disenable all buttons.
	 */
	private void setUiAsOperating(final boolean isInOperating){
		runOnUiThread(new Runnable() {
			@Override
			public void run() {
				findViewById(R.id.idBtnOpenOrClose).setEnabled(!isInOperating);
				findViewById(R.id.idBtnClear).setEnabled(!isInOperating);
				findViewById(R.id.idBtnEnroll).setEnabled(!isInOperating);
				findViewById(R.id.idBtnGetChar).setEnabled(!isInOperating);
				findViewById(R.id.idBtnGetImg).setEnabled(!isInOperating);
				findViewById(R.id.idBtnMatch).setEnabled(!isInOperating);
				findViewById(R.id.idBtnSearch).setEnabled(!isInOperating);
			}
		});
	}
	private void setUiAsOpenedOrClosed(final boolean isOpened){
		runOnUiThread(new Runnable() {
			@Override
			public void run() {
				findViewById(R.id.idBtnClear).setEnabled(isOpened);
				findViewById(R.id.idBtnEnroll).setEnabled(isOpened);
				findViewById(R.id.idBtnGetChar).setEnabled(isOpened);
				findViewById(R.id.idBtnGetImg).setEnabled(isOpened);
				findViewById(R.id.idBtnMatch).setEnabled(isOpened);
				findViewById(R.id.idBtnSearch).setEnabled(isOpened);
			}
		});
	}
	
	private void setText(final TextView tv,final String msg){
		runOnUiThread(new Runnable() {
			
			@Override
			public void run() {
				tv.setText(msg);
			}
		});
	}
}
