package com.thingmagic.rfidreader;

import android.app.ProgressDialog;
import android.content.Context;
import android.os.AsyncTask;
import android.view.View;
import android.view.inputmethod.InputMethodManager;
import android.widget.Button;
import android.widget.EditText;
import android.widget.LinearLayout;
import android.widget.RadioButton;
import android.widget.RadioGroup;
import android.widget.Spinner;
import android.widget.TableLayout;
import android.widget.TextView;

import com.thingmagic.Reader;
import com.thingmagic.ReaderConnect;
import com.thingmagic.util.LoggerUtil;
import com.thingmagic.util.Utilities;

public class ConnectionListener implements View.OnClickListener {

	static Button connectButton;
	static TextView validationField;
	static ReaderActivity readerActivity;
	static LinearLayout servicelayout;
	private static Reader reader = null;
	private static String TAG="ConnectionListener";
	private static TableLayout table ;
	private static TextView searchResultCount = null;
	public static TextView totalTagCountView = null;
//	public static TextView readrateView = null;
	private static Button readButton = null;

	// Progress Dialog
    private static ProgressDialog pDialog =null;
    private static String readerName =null;
	

	public ConnectionListener(ReaderActivity readerActivity) {
		this.readerActivity = readerActivity;
		pDialog = new ProgressDialog(readerActivity);
		pDialog.setCancelable(false);
	}

	private void findAllViewsById() {
		connectButton = (Button) readerActivity
				.findViewById(R.id.Connect_button);
		servicelayout = (LinearLayout) readerActivity
				.findViewById(R.id.ServiceLayout);
		validationField = (TextView) readerActivity
				.findViewById(R.id.ValidationField);
		table = (TableLayout) readerActivity
				.findViewById(R.id.tablelayout);
		searchResultCount = (TextView) readerActivity
				.findViewById(R.id.search_result_view);
		totalTagCountView = (TextView) readerActivity
				.findViewById(R.id.totalTagCount_view);	
//		readrateView = (TextView) readerActivity
//				.findViewById(R.id.readRate_view);
		
		readButton = (Button) readerActivity
				.findViewById(R.id.Read_button);
		
	}

	public Reader getConnectedReader() {
		return reader;
	}

	@Override
	public void onClick(View arg0) 
	{
		findAllViewsById();
		validationField.setText("");
		validationField.setVisibility(8);
		searchResultCount.setText("");
		table.removeAllViews();
		
		String query="";
		
		ReaderConnectionThread readerConnectionThread = new ReaderConnectionThread(
				query, connectButton.getText().toString());
		readerConnectionThread.execute();

	}

	private static class ReaderConnectionThread extends
			AsyncTask<Void, Void, String> {
		private String uriString = "";
		private String operation;
		private boolean operationStatus=true;

		public ReaderConnectionThread(String requestedQuery, String operation) {
			this.uriString = requestedQuery;
			this.operation = operation;

		}

		@Override
		protected void onPreExecute() {
			LoggerUtil.debug(TAG, "** onPreExecute **");	
			pDialog.setProgressStyle(ProgressDialog.STYLE_SPINNER);
			if (operation.equalsIgnoreCase("Connect")) {
				disableEdit();
				pDialog.setMessage("Connecting. Please wait...");
			}else{
				readButton.setClickable(false);
				disableEdit();
				pDialog.setMessage("DisConnecting. Please wait...");
			}
			pDialog.show();
			searchResultCount.setText("");
			
		}

		@Override
		protected String doInBackground(Void... params) {
			Long startTime = System.currentTimeMillis();
			String exception = "Exception :";
			try {
				if (operation.equalsIgnoreCase("Connect")) {
					reader = ReaderConnect.connect();
					LoggerUtil.debug(TAG, "Reader Connected");
				} else {
					reader.destroy();
					LoggerUtil.debug(TAG, "Reader Disconnected");
				}

			} catch (Exception ex) {
				operationStatus=false;
				if(ex.getMessage().contains("Connection is not created")||ex.getMessage().contains("failed to connect")){
					exception  += "Failed to connect to "+readerName;
				}else{
					exception  += ex.getMessage();	
				}				
				LoggerUtil.error(TAG, "Exception while Connecting :", ex);
			}
			return exception;
		}

		@Override
		protected void onPostExecute(String exception) {
			pDialog.dismiss();
			LoggerUtil.debug(TAG, "** onPostExecute **");			
			if (!operationStatus) {
				validationField.setText(exception);
				validationField.setVisibility(0);
				if (operation.equalsIgnoreCase("Connect")) {
					connectButton.setText("Connect");
					enableEdit();
					readerActivity.reader = null;
				}
			}
			else
			{
				validationField.setText("");
				validationField.setVisibility(8);
				if (operation.equalsIgnoreCase("Connect")) 
				{
					connectButton.setText("Disconnect");
					servicelayout.setVisibility(0);
					disableEdit();
					connectButton.setClickable(true);
					readButton.setClickable(true);
					readerActivity.reader = reader;
			    }
				else 
				{
					connectButton.setText("Connect");
					servicelayout.setVisibility(8);
					enableEdit();					
					totalTagCountView.setText("");
//					readrateView.setText("");
					readerActivity.reader = null;
				}
			}
		}
		private void disableEdit()
		{
			connectButton.setClickable(false);
		}
		private void enableEdit()
		{			
			connectButton.setClickable(true);
		}
	}

}
