package com.thingmagic.rfidreader.Listener;

import android.app.ProgressDialog;
import android.os.AsyncTask;
import android.view.View;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.TableLayout;
import android.widget.TextView;
import com.thingmagic.*;
import com.thingmagic.rfidreader.R;
import com.thingmagic.rfidreader.ReaderActivity;
import com.thingmagic.util.LoggerUtil;
import com.thingmagic.util.Utilities;

public class ConnectionListener implements View.OnClickListener {

	private static ReaderActivity mReaderActivity;
	private static Reader reader = null;
	private static LinearLayout servicelayout;
	private static TableLayout table = null;
	private static TextView validationField;
	private static TextView searchResultCount = null;
	private static TextView totalTagCountView = null;
	private static Button connectButton;
	private static Button readButton = null;
	private static ProgressDialog pDialog = null;

	private static String TAG = "ConnectionListener";
	private static String readerName = null;
	private static String readerChecked;

	public ConnectionListener(ReaderActivity readerActivity) {
		mReaderActivity = readerActivity;
		pDialog = new ProgressDialog(readerActivity);
		pDialog.setCancelable(false);
		pDialog.setProgressStyle(ProgressDialog.STYLE_SPINNER);
		findAllViewsById();
	}

	private void findAllViewsById() {
		connectButton = (Button) mReaderActivity
				.findViewById(R.id.Connect_button);
		servicelayout = (LinearLayout) mReaderActivity
				.findViewById(R.id.ServiceLayout);
		validationField = (TextView) mReaderActivity
				.findViewById(R.id.ValidationField);
		table = (TableLayout) mReaderActivity.findViewById(R.id.tablelayout);
		searchResultCount = (TextView) mReaderActivity
				.findViewById(R.id.search_result_view);
		totalTagCountView = (TextView) mReaderActivity
				.findViewById(R.id.totalTagCount_view);

		readButton = (Button) mReaderActivity.findViewById(R.id.Read_button);
	}

	@Override
	public void onClick(View arg0) {
		validationField.setText("");
		validationField.setVisibility(8);
		searchResultCount.setText("");
		totalTagCountView.setText("");
		table.removeAllViews();
		Utilities utilities = new Utilities();
		boolean validPort = true;

		ReaderConnectionThread readerConnectionThread = new ReaderConnectionThread(connectButton.getText().toString());
		readerConnectionThread.execute();
	}



	private static class ReaderConnectionThread extends
			AsyncTask<Void, Void, String> {
		private String operation;
		private boolean operationStatus = true;

		public ReaderConnectionThread(String operation) {
			this.operation = operation;

		}

		@Override
		protected void onPreExecute() {
			LoggerUtil.debug(TAG, "** onPreExecute **");			
			if (operation.equalsIgnoreCase("Connect")) {
				disableEdit();
				pDialog.setMessage("Connecting. Please wait...");
			} else {
				readButton.setClickable(false);
				disableEdit();
				pDialog.setMessage("Disconnecting. Please wait...");
			}
			pDialog.show();
			searchResultCount.setText("");

		}

		@Override
		protected String doInBackground(Void... params) {
			String exception = "Exception :";
			try {
				if (operation.equalsIgnoreCase("Connect")) {
					reader = ReaderConnect.connect();
					LoggerUtil.debug(TAG, "Reader Connected");
				} else {
					reader.destroy();
					LoggerUtil.debug(TAG, "Reader Disconnected");
				}

			} catch (Exception ex) {
				operationStatus = false;
				if (ex.getMessage().contains("Connection is not created")
						|| ex.getMessage().startsWith("Failed to connect")) {
					exception += "Failed to connect to " + readerName;
				} else {
					exception += ex.getMessage();
				}
				LoggerUtil.error(TAG, "Exception while Connecting :", ex);
			}
			return exception;
		}

		@Override
		protected void onPostExecute(String exception) {
			pDialog.dismiss();
			LoggerUtil.debug(TAG, "** onPostExecute **");
			if (!operationStatus) {
				validationField.setText(exception);
				validationField.setVisibility(0);
				totalTagCountView.setText("");
				if (operation.equalsIgnoreCase("Connect")) {
					connectButton.setText("Connect");
					enableEdit();
					mReaderActivity.reader = null;
				}
			} else {
				validationField.setText("");
				validationField.setVisibility(8);
				if (operation.equalsIgnoreCase("Connect")) {
					connectButton.setText("Disconnect");
					servicelayout.setVisibility(0);
					disableEdit();
					connectButton.setClickable(true);
					readButton.setClickable(true);
					mReaderActivity.reader = reader;
				} else {
					connectButton.setText("Connect");
					servicelayout.setVisibility(8);
					enableEdit();
					totalTagCountView.setText("");
					mReaderActivity.reader = null;
					System.gc();
				}
			}
		}

		private void disableEdit() {
			connectButton.setClickable(false);
		}

		private void enableEdit() {
			connectButton.setClickable(true);
		}
	}
}
