package com.thingmagic.rfidreader;

import java.util.ArrayList;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.res.Configuration;
import android.os.Bundle;
import android.util.Log;
import android.view.Display;
import android.view.LayoutInflater;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.ExpandableListView;
import android.widget.RadioButton;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;

import com.thingmagic.Reader;
import com.thingmagic.ReaderException;
import com.thingmagic.rfidreader.Listener.ConnectionListener;
import com.thingmagic.rfidreader.Listener.ServiceListener;
import com.thingmagic.rfidreader.customViews.ExpandableListAdapter;
import com.thingmagic.util.GlobalExceptionHandler;
import com.thingmagic.util.LoggerUtil;
import com.thingmagic.util.Utilities;


public class ReaderActivity extends Activity {
    
    private static final String TAG = "ReaderActivity";
    private RadioButton syncReadRadioButton = null;
    private RadioButton asyncReadSearchRadioButton = null;
    private static Button connectButton = null;
    private static Button readButton = null;
    private static Button clearButton = null;
    private static Button settingsButton = null;
    private static Button homeButton = null;
    private static Display display = null;
    private static TextView searchResultCount = null;
    private static TextView rowNumberLabelView = null;
    private static TextView epcDataLabelView = null;
    private static TextView epcCountLabelView = null;
    private int windowWidth;
    public int rowNumberWidth = 0;
    public int epcDataWidth = 0;
    public int epcCountWidth = 0;
    public Reader reader = null;
    private ReaderActivity activity;
    public View readOptions;
    public View performance_metrics;
    public View firmware;
    
    public ExpandableListAdapter expandableListAdapter;
    
    /**
     * The flag indicating whether content is loaded (text is shown) or not (setting view is
     * shown).
     */
    private boolean mContentLoaded;
    private View mSettingView;
    private View mDisplayView;
    
    private Spinner sp_region;
    private Button btn_region_get;
    private Button btn_region_set;
    
    @Override
    public void onCreate(Bundle savedInstanceState) {
        try {
            super.onCreate(savedInstanceState);
            setContentView(R.layout.activity_reader);
            
            mSettingView = findViewById(R.id.settings);
            mDisplayView = findViewById(R.id.displayView);
            
            // Initially hide the setting view.
            mSettingView.setVisibility(View.GONE);
            
            activity = this;
            try {
                Utilities.loadLog4j(this);
            } catch (Exception ex) {
                LoggerUtil.error(TAG, "Error loading log4j", ex);
            }
            this.findAllViewsById();
            windowWidth = display.getWidth();
            
            connectButton.setOnClickListener(new ConnectionListener(this));
            
            syncReadRadioButton.setOnClickListener(serviceRadioButtonListener);
            asyncReadSearchRadioButton.setOnClickListener(serviceRadioButtonListener);
            
            loadSettingConntentView();
            
            readButton.setOnClickListener(new ServiceListener(this));
            
            searchResultCount.setText("");
            clearButton.setOnClickListener(new ServiceListener(this).clearListener);
            
            settingsButton.setOnClickListener(new OnClickListener() {
                @Override
                public void onClick(View view) {
                    mContentLoaded = !mContentLoaded;
                    showSettingsOrTagsView(mContentLoaded);
                }
            });
            
            setPortraitTableLayout();
            System.gc();
            Thread.currentThread().setDefaultUncaughtExceptionHandler(new GlobalExceptionHandler(activity));
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
    
    private void loadSettingConntentView() {
        ExpandableListView expandableList = (ExpandableListView) findViewById(R.id.expandableListView1);
        
        LayoutInflater infalInflater = (LayoutInflater) getSystemService(Context.LAYOUT_INFLATER_SERVICE);
        readOptions = infalInflater.inflate(R.layout.settings_read_options, null);
        performance_metrics = infalInflater.inflate(R.layout.performance_metrics, null);
        firmware = infalInflater.inflate(R.layout.firmware, null);
        ArrayList<View> ChildViews = new ArrayList<View>();
        ChildViews.add(readOptions);
        ChildViews.add(performance_metrics);
        ChildViews.add(firmware);
        
        String[] groups = {"Read Options", "Statistics", "Firmware"};
        expandableListAdapter = new ExpandableListAdapter(this, groups, ChildViews);
        
        // Set this blank adapter to the list view
        expandableList.setAdapter(expandableListAdapter);
        homeButton.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View view) {
                mContentLoaded = !mContentLoaded;
                showSettingsOrTagsView(mContentLoaded);
            }
        });
        sp_region = (Spinner) findViewById(R.id.sp_region);
        btn_region_get = (Button) findViewById(R.id.btn_region_get);
        btn_region_set = (Button) findViewById(R.id.btn_region_set);
        btn_region_get.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                Reader.Region region = getRegion();
                int position = -1;
                if (region != null) {
                    switch (region) {
                        case NA2:
                            position = 1;
                            break;
                        case IN:
                            position = 2;
                            break;
                        case JP:
                            position = 3;
                            break;
                        case PRC:
                            position = 4;
                            break;
                        case EU3:
                            position = 5;
                            break;
                        case KR2:
                            position = 6;
                            break;
                        case AU:
                            position = 7;
                            break;
                        case NZ:
                            position = 8;
                            break;
                    }
                }
                sp_region.setSelection(position);
            }
        });
        btn_region_set.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View v) {
                Reader.Region region = null;
                switch (sp_region.getSelectedItemPosition()) {
                    case 1://北美
                        region = Reader.Region.NA2;
                        break;
                    case 2://印度
                        region = Reader.Region.IN;
                        break;
                    case 3://日本
                        region = Reader.Region.JP;
                        break;
                    case 4://中国
                        region = Reader.Region.PRC;
                        break;
                    case 5://欧洲
                        region = Reader.Region.EU3;
                        break;
                    case 6://韩国
                        region = Reader.Region.KR2;
                        break;
                    case 7://澳大利亚
                        region = Reader.Region.AU;
                        break;
                    case 8://新西兰
                        region = Reader.Region.NZ;
                        break;
                }
                if (region != null) {
                    setRegion(region);
                }
            }
        });
    }
    
    
    /**
     * 设置区域
     *
     * @param region
     */
    private void setRegion(Reader.Region region) {
        try {
            if (reader != null) {
                reader.paramSet("/reader/region/id", region);
                Toast.makeText(this, "set success", Toast.LENGTH_SHORT).show();
            } else {
                Toast.makeText(this, "set failed,check connect state", Toast.LENGTH_SHORT).show();
            }
        } catch (ReaderException e) {
            e.printStackTrace();
            Toast.makeText(this, "set failed", Toast.LENGTH_SHORT).show();
        }
    }
    
    /**
     * 读取区域
     *
     * @return
     */
    private Reader.Region getRegion() {
        Reader.Region region = null;
        try {
            if (reader != null) {
                region = (Reader.Region) reader.paramGet("/reader/region/id");
                Toast.makeText(this, "get success", Toast.LENGTH_SHORT).show();
            } else {
                Toast.makeText(this, "get failed,check connect state", Toast.LENGTH_SHORT).show();
            }
        } catch (ReaderException e) {
            e.printStackTrace();
            Toast.makeText(this, "get failed", Toast.LENGTH_SHORT).show();
        }
        return region;
    }
    
    /**
     * Cross-fades between {@link #mSettingView} and {@link #mDisplayView}.
     */
    private void showSettingsOrTagsView(boolean contentLoaded) {
        // Decide which view to hide and which to show.
        final View showView = contentLoaded ? mSettingView : mDisplayView;
        final View hideView = contentLoaded ? mDisplayView : mSettingView;
        
        showView.setVisibility(View.VISIBLE);
        showView.setVisibility(View.KEEP_SCREEN_ON);
        hideView.setVisibility(View.GONE);
    }
    
    /**
     * This is called when the screen rotates.
     * (onCreate is no longer called when screen rotates due to manifest, see: android:configChanges)
     */
    @Override
    public void onConfigurationChanged(Configuration newConfig) {
        super.onConfigurationChanged(newConfig);
        int orientation = newConfig.orientation;
        if (orientation == Configuration.ORIENTATION_PORTRAIT) {
            setPortraitTableLayout();
        } else if (orientation == Configuration.ORIENTATION_LANDSCAPE) {
            setPortraitTableLayout();
        }
    }
    
    private void setPortraitTableLayout() {
        rowNumberLabelView.setWidth((int) (windowWidth * (0.1)));
        epcDataLabelView.setWidth((int) (windowWidth * (0.73)));
        epcCountLabelView.setWidth((int) (windowWidth * (0.17)));
        rowNumberWidth = (int) (windowWidth * (0.09));
        epcDataWidth = (int) (windowWidth * (0.75));
        epcCountWidth = (int) (windowWidth * (0.16));
    }
    
    @Override
    public void onBackPressed() {
        String message = "";
        String title = "";
        if (reader != null) {
            title = "Run in background";
            message = "Are you sure you want to run this application in background?";
        } else {
            title = "Closing application";
            message = "Are you sure you want to close this application?";
        }
        
        new AlertDialog.Builder(this)
                .setIcon(android.R.drawable.ic_dialog_alert)
                .setTitle(title)
                .setMessage(message)
                .setPositiveButton("Yes", new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        if (reader != null) {
                            moveTaskToBack(true);
                        } else {
                            System.gc();
                            finish();
                        }
                    }
                })
                .setNegativeButton("No", null)
                .setCancelable(false)
                .show();
    }
    
    private void findAllViewsById() {
        connectButton = (Button) findViewById(R.id.Connect_button);
        syncReadRadioButton = (RadioButton) findViewById(R.id.SyncRead_radio_button);
        asyncReadSearchRadioButton = (RadioButton) findViewById(R.id.AsyncRead_radio_button);
        readButton = (Button) findViewById(R.id.Read_button);
        searchResultCount = (TextView) findViewById(R.id.search_result_view);
        clearButton = (Button) findViewById(R.id.Clear);
        settingsButton = (Button) findViewById(R.id.settings_view);
        display = ((android.view.WindowManager) getSystemService(Context.WINDOW_SERVICE)).getDefaultDisplay();
        rowNumberLabelView = (TextView) findViewById(R.id.SNOLabel);
        epcDataLabelView = (TextView) findViewById(R.id.EPCLabel);
        epcCountLabelView = (TextView) findViewById(R.id.COUNTLabel);
        homeButton = (Button) findViewById(R.id.btn_back_main);
    }
    
    private OnClickListener serviceRadioButtonListener = new OnClickListener() {
        public void onClick(View v) {
            RadioButton radioButton = (RadioButton) v;
            if (radioButton.getText().equals("AsyncRead")) {
                readButton.setText("Start Reading");
            } else if (readButton.getText().equals("Start Reading")) {
                readButton.setText("Read");
            }
        }
    };
}