package com.thomas;

import android.app.Activity;
import android.os.Bundle;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.Toast;

import com.senter.iot.support.openapi.uhf.UhfH;


public class FrequencySettingsActivity extends Activity {
    
    Spinner SpinnerFreMode;
    EditText EtFreStart;
    EditText EtFreEnd;
    Button BtSetFrequency;
    Button BtGetFrequency;
    
    private UhfH uhfH;
    
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.frequencysettings);
        uhfH = UhfH.getInstance();
        
        SpinnerFreMode = (Spinner) findViewById(R.id.SpinnerFreMode);
        EtFreStart = (EditText) findViewById(R.id.EtFreStart);
        EtFreEnd = (EditText) findViewById(R.id.EtFreEnd);
        BtSetFrequency = (Button) findViewById(R.id.BtSetFrequency);
        BtGetFrequency = (Button) findViewById(R.id.BtGetFrequency);
        
        BtSetFrequency.setOnClickListener(new BtSetFrequencyClickListener());
        BtGetFrequency.setOnClickListener(new BtGetFrequencyClickListener());
    }
    
    class BtSetFrequencyClickListener implements OnClickListener {
        
        @Override
        public void onClick(View arg0) {
            UhfH.FrequencyMode mode = null;
            boolean isSuccess = false;
            switch (SpinnerFreMode.getSelectedItemPosition()) {
                case 0:
                    mode = UhfH.FrequencyMode.Chinese_920_925MHz;
                    break;
                case 1:
                    mode = UhfH.FrequencyMode.Chinese_840_845MHz;
                    break;
                case 2:
                    mode = UhfH.FrequencyMode.ETSI;
                    break;
                case 3:
                    mode = UhfH.FrequencyMode.Fixed;
                    break;
                case 4:
                    mode = UhfH.FrequencyMode.FCC_902_928MHz;
                    break;
                case 5:
                    mode = UhfH.FrequencyMode.Taiwan_922_928MHz;
                    break;
                case 6:
                    mode = UhfH.FrequencyMode.Korea_917_921MHz;
                    break;
                case 7:
                    mode = UhfH.FrequencyMode.Brazil_902_907MHz;
                    break;
                case 8:
                    mode = UhfH.FrequencyMode.Brazil_915_928MHz;
                    break;
                default:
                    mode = UhfH.FrequencyMode.User_Define;
                    break;
            }
            if (mode != UhfH.FrequencyMode.User_Define) {
                UhfH.Frequency frequency = mode.getStandardFrequency();
                isSuccess = uhfH.setFrequency(frequency);
            } else {//用户自定义
                String freStartStr = EtFreStart.getText().toString().trim();
                if (!Function.isDecimal(freStartStr)) {
                    Toast.makeText(FrequencySettingsActivity.this, "起始频率 请输入10进制数据", Toast.LENGTH_SHORT).show();
                    return;
                }
                int _freStr = Integer.parseInt(freStartStr);
                if (_freStr < 840) {
                    Toast.makeText(FrequencySettingsActivity.this, "起始频率 应不小于840MHz", Toast.LENGTH_SHORT).show();
                    return;
                }
                String freEndStr = EtFreEnd.getText().toString().trim();
                if (!Function.isDecimal(freEndStr)) {
                    Toast.makeText(FrequencySettingsActivity.this, "终止频率 请输入10进制数据", Toast.LENGTH_SHORT).show();
                    return;
                }
                int freEnd = Integer.parseInt(freEndStr);
                if (freEnd > 960) {
                    Toast.makeText(FrequencySettingsActivity.this, "终止频率 应不大于960MHz", Toast.LENGTH_SHORT).show();
                    return;
                }
                if (freEnd < _freStr) {
                    Toast.makeText(FrequencySettingsActivity.this, "终止频率 不应小于 起始频率", Toast.LENGTH_SHORT).show();
                    return;
                }
                if (freEnd - _freStr > 32) {
                    Toast.makeText(FrequencySettingsActivity.this, "频率带宽 不能大于32MHz", Toast.LENGTH_SHORT).show();
                    return;
                }
                isSuccess = uhfH.setUserDefineFrequency(_freStr, freEnd);
            }
            if (isSuccess) {
                Toast.makeText(FrequencySettingsActivity.this, "设置频率成功", Toast.LENGTH_SHORT).show();
                ReadFrequency();
            } else {
                Toast.makeText(FrequencySettingsActivity.this, "设置频率失败", Toast.LENGTH_SHORT).show();
            }
        }
    }
    
    private void ReadFrequency() {
        
        String frequencyStr = "";
        UhfH.Frequency frequency = uhfH.getFrequency();
        if (frequency != null) {
            frequencyStr = frequency.getFrequencyStr();
            Toast.makeText(FrequencySettingsActivity.this, "读取频率成功，频率为：" + frequencyStr + " MHz", Toast.LENGTH_SHORT).show();
            
            UhfH.FrequencyMode freMode = frequency.getMode();
            
            if (freMode == UhfH.FrequencyMode.Chinese_920_925MHz) {
                SpinnerFreMode.setSelection(0);
            } else if (freMode == UhfH.FrequencyMode.Chinese_840_845MHz) {
                SpinnerFreMode.setSelection(1);
            } else if (freMode == UhfH.FrequencyMode.ETSI) {
                SpinnerFreMode.setSelection(2);
            } else if (freMode == UhfH.FrequencyMode.Fixed) {
                SpinnerFreMode.setSelection(3);
            } else {
                SpinnerFreMode.setSelection(9);
            }
        } else {
            Toast.makeText(FrequencySettingsActivity.this, "读取频率失败", Toast.LENGTH_SHORT).show();
        }
    }
    
    class BtGetFrequencyClickListener implements OnClickListener {
        
        @Override
        public void onClick(View arg0) {
            
            ReadFrequency();
        }
    }
    
}
