package com.thomas;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;

import android.app.Activity;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.RadioButton;
import android.widget.RadioGroup;
import android.widget.RadioGroup.OnCheckedChangeListener;
import android.widget.SimpleAdapter;
import android.widget.Toast;

import com.senter.iot.support.openapi.uhf.UhfH;


public class InventoryActivity extends Activity {
    
    private static int inventoryFlag;
    static ArrayList<HashMap<String, String>> tagList;
    SimpleAdapter adapter;
    
    Button BtClear;
    EditText EtCountOfTags;
    RadioGroup RgInventory;
    RadioButton RbInventorySingle;
    RadioButton RbInventorySingleLoop;
    RadioButton RbInventoryMultiLoop;
    Button BtInventory;
    ListView LvTags;
    
    private UhfH uhfH;
    
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.inventory);
        uhfH = UhfH.getInstance();
        inventoryFlag = 1;
        tagList = new ArrayList<HashMap<String, String>>();
        adapter = new SimpleAdapter(InventoryActivity.this,
                                    tagList,
                                    R.layout.listtag_items,
                                    new String[]{"tagUii", "tagLen", "tagCount"},
                                    new int[]{R.id.TvTagUii, R.id.TvTagLen, R.id.TvTagCount});
        
        BtClear = (Button) findViewById(R.id.BtClear);
        EtCountOfTags = (EditText) findViewById(R.id.EtCountOfTags);
        RgInventory = (RadioGroup) findViewById(R.id.RgInventory);
        RbInventorySingle = (RadioButton) findViewById(R.id.RbInventorySingle);
        RbInventorySingleLoop = (RadioButton) findViewById(R.id.RbInventorySingleLoop);
        RbInventoryMultiLoop = (RadioButton) findViewById(R.id.RbInventoryMultiLoop);
        BtInventory = (Button) findViewById(R.id.BtInventory);
        LvTags = (ListView) findViewById(R.id.LvTags);
        
        BtClear.setOnClickListener(new BtClearClickListener());
        EtCountOfTags.setKeyListener(null);
        RgInventory.setOnCheckedChangeListener(new RgInventoryCheckedListener());
        BtInventory.setOnClickListener(new BtInventoryClickListener());
    }
    
    public class BtClearClickListener implements OnClickListener {
        
        @Override
        public void onClick(View arg0) {
            
            EtCountOfTags.setText("");
            
            tagList.clear();
            adapter.notifyDataSetChanged();
        }
    }
    
    public class RgInventoryCheckedListener implements OnCheckedChangeListener {
        
        @Override
        public void onCheckedChanged(RadioGroup arg0, int arg1) {
            if (arg1 == RbInventorySingle.getId()) {
                //单步识别
                inventoryFlag = 0;
            } else if (arg1 == RbInventorySingleLoop.getId()) {
                //单标签循环识别
                inventoryFlag = 1;
            } else {
                //多标签循环识别
                inventoryFlag = 3;
            }
        }
    }
    
    public class BtInventoryClickListener implements OnClickListener {
        
        @Override
        public void onClick(View arg0) {
            if (BtInventory.getText().equals("识别标签"))//识别标签
            {
                if (inventoryFlag == 0) {
                    UhfH.UII uii = uhfH.inventorySingleTagOnce();
                    int index = -1;
                    if (uii != null) {
                        String uiiStr = Function.bytesToHexString(uii.getBytes(), uii.getUiiLength());
                        HashMap<String, String> map = new HashMap<String, String>();
                        map.put("tagUii", uiiStr);
                        map.put("tagLen", String.valueOf(uii.getUiiLength()));
                        map.put("tagCount", String.valueOf(1));
                        index = checkIsExist(uiiStr, tagList);
                        if (index == -1) {
                            tagList.add(map);
                            LvTags.setAdapter(adapter);
                        } else {
                            int tagcount = Integer.parseInt(tagList.get(index).get("tagCount"), 10) + 1;
                            
                            map.put("tagCount", String.valueOf(tagcount));
                            tagList.set(index, map);
                            adapter.notifyDataSetChanged();
                        }
                        EtCountOfTags.setText("" + adapter.getCount());
                    } else {
                        Toast.makeText(InventoryActivity.this, "单步识别失败", Toast.LENGTH_SHORT).show();
                    }
                } else if (inventoryFlag == 1) {
                    boolean isSuccess = uhfH.startInventoryLoopSingleTag(new UhfH.OnNewUiiInventoried() {
                        @Override
                        public void onNewUiiReceived(UhfH.UII uii) {
                            Message msg = handler.obtainMessage();
                            Bundle bundle = new Bundle();
                            bundle.putString("tagUii", Function.bytesToHexString(uii.getBytes(), uii.getUiiLength()));
                            bundle.putInt("tagLen", uii.getUiiLength());
                            msg.setData(bundle);
                            handler.sendMessage(msg);
                        }
                    });
                    if (isSuccess) {
                        BtInventory.setText("停止识别");
                    } else {
                        uhfH.stopLoopOperate();
                        Toast.makeText(InventoryActivity.this, "开启单标签循环识别失败", Toast.LENGTH_SHORT).show();
                    }
                    
                } else {
                    boolean isSuccess = uhfH.startInventoryLoopMultiTag(new UhfH.OnNewUiiInventoried() {
                        @Override
                        public void onNewUiiReceived(UhfH.UII uii) {
                            Message msg = handler.obtainMessage();
                            Bundle bundle = new Bundle();
                            bundle.putString("tagUii", Function.bytesToHexString(uii.getBytes(), uii.getUiiLength()));
                            bundle.putInt("tagLen", uii.getUiiLength());
                            msg.setData(bundle);
                            handler.sendMessage(msg);
                        }
                    });
                    if (isSuccess) {
                        BtInventory.setText("停止识别");
                    } else {
                        uhfH.stopLoopOperate();
                        Toast.makeText(InventoryActivity.this, "开启多标签识别失败", Toast.LENGTH_SHORT).show();
                    }
                }
            } else {//停止识别
                uhfH.stopLoopOperate();
                BtInventory.setText("识别标签");
            }
        }
    }
    
    public int checkIsExist(String uiiStr, ArrayList<HashMap<String, String>> tagList) {
        int existFlag = -1;
        String tempStr = "";
        for (int i = 0; i < tagList.size(); i++) {
            HashMap<String, String> temp = new HashMap<String, String>();
            temp = tagList.get(i);
            
            tempStr = temp.get("tagUii");
            
            if (uiiStr != "" && uiiStr.equals(tempStr)) {
                existFlag = i;
            }
        }
        return existFlag;
    }
    
    Handler handler = new Handler() {
        
        @Override
        public void handleMessage(Message msg) {
            
            Bundle bundle = msg.getData();
            String tagUii = bundle.getString("tagUii");
            int tagLen = bundle.getInt("tagLen");
            
            HashMap<String, String> map = new HashMap<String, String>();
            
            map.put("tagUii", tagUii);
            map.put("tagLen", String.valueOf(tagLen));
            map.put("tagCount", String.valueOf(1));
            
            boolean bool = false;
            int index = -1;
            try {
                bool = (tagUii != "") && (!tagUii.equals(null));
            } catch (Exception e) {
                Log.v("BreakPoint", "异常：" + e.getMessage());
                e.printStackTrace();
            }
            
            if (bool) {
                index = checkIsExist(tagUii, tagList);
                
                if (index == -1) {
                    tagList.add(map);
                    LvTags.setAdapter(adapter);
                    EtCountOfTags.setText("" + adapter.getCount());
                } else {
                    int tagcount = Integer.parseInt(tagList.get(index).get("tagCount"), 10) + 1;
                    
                    map.put("tagCount", String.valueOf(tagcount));
                    
                    tagList.set(index, map);
                    adapter.notifyDataSetChanged();
                }
            }
        }
    };
    
    
    @Override
    public void onBackPressed() {
        uhfH.stopLoopOperate();
        super.onBackPressed();
    }
}
