package com.thomas;

import android.app.Activity;
import android.os.Bundle;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Toast;

import com.senter.iot.support.openapi.uhf.UhfH;


public class RegisterSettingsActivity extends Activity {
    
    EditText EtAddrReg;
    EditText EtLenReg;
    EditText EtDataReg;
    Button BtSetRegister;
    Button BtGetRegister;
    Button BtSaveRegister;
    Button BtResetRegister;
    
    private UhfH uhfH;
    
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.registersettings);
        uhfH = UhfH.getInstance();
        
        EtAddrReg = (EditText) findViewById(R.id.EtAddrReg);
        EtLenReg = (EditText) findViewById(R.id.EtLenReg);
        EtDataReg = (EditText) findViewById(R.id.EtDataReg);
        BtSetRegister = (Button) findViewById(R.id.BtSetRegister);
        BtGetRegister = (Button) findViewById(R.id.BtGetRegister);
        BtSaveRegister = (Button) findViewById(R.id.BtSaveRegister);
        BtResetRegister = (Button) findViewById(R.id.BtResetRegister);
        
        BtSetRegister.setOnClickListener(new BtSetRegisterClickListener());
        BtGetRegister.setOnClickListener(new BtGetRegisterClickListener());
        BtSaveRegister.setOnClickListener(new BtSaveRegisterClickListener());
        BtResetRegister.setOnClickListener(new BtResetRegisterClickListener());
    }
    
    class BtSetRegisterClickListener implements OnClickListener {
        
        @Override
        public void onClick(View arg0) {
            int addr = 0;
            int len = 0;
            String addrRegStr = EtAddrReg.getText().toString().trim();
            if (addrRegStr.equals("")) {
                Toast.makeText(RegisterSettingsActivity.this, "地址 不能为空", Toast.LENGTH_SHORT).show();
                return;
            }
            if (!Function.isHex(addrRegStr)) {
                Toast.makeText(RegisterSettingsActivity.this, "地址 请输入16进制数据", Toast.LENGTH_SHORT).show();
                return;
            }
            addr = Integer.parseInt(addrRegStr, 16);
            
            String lenRegStr = EtLenReg.getText().toString().trim();
            if (lenRegStr.equals("")) {
                Toast.makeText(RegisterSettingsActivity.this, "长度 不能为空", Toast.LENGTH_SHORT).show();
                return;
            }
            if (!Function.isDecimal(lenRegStr)) {
                Toast.makeText(RegisterSettingsActivity.this, "长度 请输入10进制数据", Toast.LENGTH_SHORT).show();
                return;
            }
            len = Integer.parseInt(lenRegStr);
            
            String valueStr = EtDataReg.getText().toString().trim();
            if (valueStr.equals("")) {
                Toast.makeText(RegisterSettingsActivity.this, "数据 不能为空", Toast.LENGTH_SHORT).show();
                return;
            }
            if (!Function.isHex(valueStr) || valueStr.length() != 2 * len) {
                Toast.makeText(RegisterSettingsActivity.this, "数据 请输入指定长度的16进制数据", Toast.LENGTH_SHORT).show();
                return;
            }
            boolean isSuccess = uhfH.setRegister(addr, len, Function.HexStringToBytes(valueStr));
            if (isSuccess) {
                Toast.makeText(RegisterSettingsActivity.this, "设置寄存器 成功", Toast.LENGTH_SHORT).show();
            } else {
                Toast.makeText(RegisterSettingsActivity.this, "设置寄存器 失败", Toast.LENGTH_SHORT).show();
            }
        }
    }
    
    class BtGetRegisterClickListener implements OnClickListener {
        
        @Override
        public void onClick(View arg0) {
            int addr = 0;
            int len = 0;
            
            String addrRegStr = EtAddrReg.getText().toString().trim();
            if (addrRegStr.equals("")) {
                Toast.makeText(RegisterSettingsActivity.this, "地址 不能为空", Toast.LENGTH_SHORT).show();
                return;
            }
            if (!Function.isHex(addrRegStr)) {
                Toast.makeText(RegisterSettingsActivity.this, "地址 请输入16进制数据", Toast.LENGTH_SHORT).show();
                return;
            }
            addr = Integer.parseInt(addrRegStr, 16);
            
            String lenRegStr = EtLenReg.getText().toString().trim();
            if (lenRegStr.equals("")) {
                Toast.makeText(RegisterSettingsActivity.this, "长度 不能为空", Toast.LENGTH_SHORT).show();
                return;
            }
            if (!Function.isDecimal(lenRegStr)) {
                Toast.makeText(RegisterSettingsActivity.this, "长度 请输入10进制数据", Toast.LENGTH_SHORT).show();
                return;
            }
            len = Integer.parseInt(lenRegStr);
            byte[] data = uhfH.getRegister(addr, len);
            if (data != null) {
                String dataStr = Function.bytesToHexString(data, data.length);
                EtDataReg.setText(dataStr);
                Toast.makeText(RegisterSettingsActivity.this, "读取寄存器 成功", Toast.LENGTH_SHORT).show();
            } else {
                Toast.makeText(RegisterSettingsActivity.this, "读取寄存器 失败", Toast.LENGTH_SHORT).show();
            }
        }
    }
    
    class BtSaveRegisterClickListener implements OnClickListener {
        
        @Override
        public void onClick(View arg0) {
            boolean isSuccess = uhfH.saveRegister();
            if (isSuccess) {
                Toast.makeText(RegisterSettingsActivity.this, "保存寄存器 成功", Toast.LENGTH_SHORT).show();
            } else {
                Toast.makeText(RegisterSettingsActivity.this, "保存寄存器 失败", Toast.LENGTH_SHORT).show();
            }
        }
    }
    
    class BtResetRegisterClickListener implements OnClickListener {
        
        @Override
        public void onClick(View arg0) {
            boolean isSuccess = uhfH.resetRegister();
            if (isSuccess) {
                Toast.makeText(RegisterSettingsActivity.this, "恢复寄存器 成功", Toast.LENGTH_SHORT).show();
            } else {
                Toast.makeText(RegisterSettingsActivity.this, "恢复寄存器 失败", Toast.LENGTH_SHORT).show();
            }
        }
    }
    
}
