package com.thomas;

import java.util.Random;

import android.app.Activity;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.Toast;

import com.senter.iot.support.openapi.uhf.UhfH;


public class SafetyManageActivity extends Activity {
    Spinner SpinnerKeyType;
    Spinner SpinnerAdf;
    EditText EtMasterKey;
    EditText EtAdfKey;
    EditText EtNewKey;
    Button BtDownloadKey;
    Button BtSaveKey;
    Button BtSetAdf;
    Button BtGetAdf;
    
    private UhfH uhfH;
    
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.safemanage);
        uhfH = UhfH.getInstance();
        
        SpinnerKeyType = (Spinner) findViewById(R.id.SpinnerKeyType);
        SpinnerAdf = (Spinner) findViewById(R.id.SpinnerAdf);
        EtMasterKey = (EditText) findViewById(R.id.EtMasterKey);
        EtAdfKey = (EditText) findViewById(R.id.EtAdfKey);
        EtNewKey = (EditText) findViewById(R.id.EtNewKey);
        BtDownloadKey = (Button) findViewById(R.id.BtDownloadKey);
        BtSaveKey = (Button) findViewById(R.id.BtSaveKey);
        BtSetAdf = (Button) findViewById(R.id.BtSetAdf);
        BtGetAdf = (Button) findViewById(R.id.BtGetAdf);
        
        BtDownloadKey.setOnClickListener(new BtDownloadKeyClickListener());
        BtSaveKey.setOnClickListener(new BtSaveKeyClickListener());
        BtSetAdf.setOnClickListener(new BtSetAdfClickListener());
        BtGetAdf.setOnClickListener(new BtGetAdfClickListener());
    }
    
    class BtDownloadKeyClickListener implements OnClickListener {
        
        @Override
        public void onClick(View arg0) {
            int i = 0;
            byte[] RNm = new byte[8];
            byte[] RN = new byte[16];
            String RNmStr = "";
            String _RNmStr = "";
            
            UhfH.KeyType keyType = UhfH.KeyType.ValueOf(SpinnerKeyType.getSelectedItemPosition());
            
            //获取ADF应用环境
            UhfH.ApplicationDataFile adf = uhfH.getADF();
            if (adf == null) {
                Toast.makeText(SafetyManageActivity.this, "获取 ADF 失败", Toast.LENGTH_SHORT).show();
                return;
            }
            //新密钥
            String keyStr = EtNewKey.getText().toString().trim();
            if (!Function.isHex(keyStr)) {
                Toast.makeText(SafetyManageActivity.this, "更改密钥 请输入16进制数据", Toast.LENGTH_SHORT).show();
                return;
            }
            if (keyType == UhfH.KeyType.master_key || keyType == UhfH.KeyType.application_maintenance_key || keyType == UhfH.KeyType.application_master_key) {
                if (keyStr.length() != 32) {
                    Toast.makeText(SafetyManageActivity.this, "更改密钥 请输入32个字符的16进制数据", Toast.LENGTH_SHORT).show();
                    return;
                }
            } else {
                if (keyStr.length() != 8) {
                    Toast.makeText(SafetyManageActivity.this, "更改密钥 请输入8个字符的16进制数据", Toast.LENGTH_SHORT).show();
                    return;
                }
            }
            
            byte[] key = Function.HexStringToBytes(keyStr);
            
            if (adf != UhfH.ApplicationDataFile.ADF0 || keyType == UhfH.KeyType.master_key) { //如果应用环境是ADF0，则不用执行鉴别操作
                //用于获取管理权限
                String key_manageStr = EtMasterKey.getText().toString().trim();
                if (key_manageStr.length() != 32 || !Function.isHex(key_manageStr)) {
                    Toast.makeText(SafetyManageActivity.this, "主控密钥 请输入32个字符的16进制数据", Toast.LENGTH_SHORT).show();
                    return;
                }
                byte[] RNr = uhfH.identifyRequest(UhfH.IdentifyOption.manage_permission);
                if (RNr == null) {
                    Toast.makeText(SafetyManageActivity.this, "鉴别请求 失败", Toast.LENGTH_SHORT).show();
                    return;
                }
                
                byte[] key_manage = Function.HexStringToBytes(key_manageStr);
                
                GenerateRandom(RNm);
                for (i = 0; i < 8; i++) {
                    RN[i] = RNr[i];
                }
                for (i = 0; i < 8; i++) {
                    RN[8 + i] = RNm[i];
                }
                
                byte[] token1 = uhfH.encrypt(key_manage, RN);
                byte[] token2 = uhfH.identifyToken1(UhfH.IdentifyOption.manage_permission, token1);
                if (token2 == null) {
                    Toast.makeText(SafetyManageActivity.this, "鉴别 token1 失败", Toast.LENGTH_SHORT).show();
                    return;
                }
                byte[] _rn = uhfH.decrypt(key_manage, token2);
                _RNmStr = Function.bytesToHexString(_rn, _rn.length).substring(16);
                RNmStr = Function.bytesToHexString(RNm, 8);
                
                Log.d("mine", "RNmStr-->" + RNmStr);
                Log.d("mine", "_RNmStr->" + _RNmStr);
                
                if (!RNmStr.equals(_RNmStr)) {
                    Toast.makeText(SafetyManageActivity.this, "鉴别 失败", Toast.LENGTH_SHORT).show();
                    if (!uhfH.identifyFinish(0)) {
                        Toast.makeText(SafetyManageActivity.this, "鉴别结束 失败", Toast.LENGTH_SHORT).show();
                    } else {
                        Toast.makeText(SafetyManageActivity.this, "鉴别结束", Toast.LENGTH_SHORT).show();
                    }
                    return;
                } else {
                    if (!uhfH.identifyFinish(1)) {
                        Toast.makeText(SafetyManageActivity.this, "鉴别结束 失败", Toast.LENGTH_SHORT).show();
                        return;
                    } else {
                        Toast.makeText(SafetyManageActivity.this, "鉴别结束", Toast.LENGTH_SHORT).show();
                    }
                }
                
                //再获取 应用环境密钥下载权限 或者 主控密钥下载权限
                if (keyType == UhfH.KeyType.master_key) {
                    RNr = uhfH.identifyRequest(UhfH.IdentifyOption.main_control_key_download_permission);
                    if (RNr == null) {
                        Toast.makeText(SafetyManageActivity.this, "鉴别请求 失败", Toast.LENGTH_SHORT).show();
                        return;
                    }
                    GenerateRandom(RNm);
                    for (i = 0; i < 8; i++) {
                        RN[i] = (byte) RNr[i];
                    }
                    for (i = 0; i < 8; i++) {
                        RN[8 + i] = RNm[i];
                    }
                    
                    token1 = uhfH.encrypt(key_manage, RN);
                    token2 = uhfH.identifyToken1(UhfH.IdentifyOption.application_environment_key_download_permission, token1);
                    if (token2 == null) {
                        Toast.makeText(SafetyManageActivity.this, "鉴别 token1 失败", Toast.LENGTH_SHORT).show();
                        return;
                    }
                    _rn = uhfH.decrypt(key_manage, token2);
                    _RNmStr = Function.bytesToHexString(_rn, _rn.length).substring(16);
                    RNmStr = Function.bytesToHexString(RNm, 8);
                    if (!RNmStr.equals(_RNmStr)) {
                        Toast.makeText(SafetyManageActivity.this, "鉴别 失败", Toast.LENGTH_SHORT).show();
                        if (!uhfH.identifyFinish(0)) {
                            Toast.makeText(SafetyManageActivity.this, "鉴别结束 失败", Toast.LENGTH_SHORT).show();
                        } else {
                            Toast.makeText(SafetyManageActivity.this, "鉴别结束", Toast.LENGTH_SHORT).show();
                        }
                        return;
                    } else {
                        if (!uhfH.identifyFinish(1)) {
                            Toast.makeText(SafetyManageActivity.this, "鉴别结束 失败", Toast.LENGTH_SHORT).show();
                            return;
                        } else {
                            Toast.makeText(SafetyManageActivity.this, "鉴别结束", Toast.LENGTH_SHORT).show();
                        }
                    }
                } else {
                    String key_adfStr = EtAdfKey.getText().toString().trim();
                    if (key_adfStr.length() != 32 || !Function.isHex(key_adfStr)) {
                        Toast.makeText(SafetyManageActivity.this, "应用环境密钥 请输入32个字符的16进制数据", Toast.LENGTH_SHORT).show();
                        return;
                    }
                    
                    RNr = uhfH.identifyRequest(UhfH.IdentifyOption.application_environment_key_download_permission);
                    
                    if (RNr == null) {
                        Toast.makeText(SafetyManageActivity.this, "鉴别请求 失败", Toast.LENGTH_SHORT).show();
                        return;
                    }
                    
                    byte[] key_ad = Function.HexStringToBytes(key_adfStr);
                    
                    GenerateRandom(RNm);
                    for (i = 0; i < 8; i++) {
                        RN[i] = (byte) RNr[i];
                    }
                    for (i = 0; i < 8; i++) {
                        RN[8 + i] = RNm[i];
                    }
                    
                    token1 = uhfH.encrypt(key_ad, RN);
                    
                    token2 = uhfH.identifyToken1(UhfH.IdentifyOption.application_environment_key_download_permission, token1);
                    
                    if (token2 == null) {
                        Toast.makeText(SafetyManageActivity.this, "鉴别 token1 失败", Toast.LENGTH_SHORT).show();
                        return;
                    }
                    _rn = uhfH.decrypt(key_ad, token2);
                    _RNmStr = Function.bytesToHexString(_rn, _rn.length).substring(16);
                    RNmStr = Function.bytesToHexString(RNm, 8);
                    if (RNmStr.equals(_RNmStr)) {
                        Toast.makeText(SafetyManageActivity.this, "鉴别 失败", Toast.LENGTH_SHORT).show();
                        if (!uhfH.identifyFinish(0)) {
                            Toast.makeText(SafetyManageActivity.this, "鉴别结束 失败", Toast.LENGTH_SHORT).show();
                        } else {
                            Toast.makeText(SafetyManageActivity.this, "鉴别结束", Toast.LENGTH_SHORT).show();
                        }
                        return;
                    } else {
                        if (!uhfH.identifyFinish(1)) {
                            Toast.makeText(SafetyManageActivity.this, "鉴别结束 失败", Toast.LENGTH_SHORT).show();
                            return;
                        } else {
                            Toast.makeText(SafetyManageActivity.this, "鉴别结束", Toast.LENGTH_SHORT).show();
                        }
                    }
                }
            }
            
            if (!uhfH.keyDownload(keyType, key)) {
                Toast.makeText(SafetyManageActivity.this, "密钥下载 失败", Toast.LENGTH_SHORT).show();
                return;
            } else {
                Toast.makeText(SafetyManageActivity.this, "密钥下载 成功", Toast.LENGTH_SHORT).show();
            }
        }
    }
    
    class BtSaveKeyClickListener implements OnClickListener {
        
        @Override
        public void onClick(View arg0) {
            UhfH.KeyType keyType = UhfH.KeyType.ValueOf(SpinnerKeyType.getSelectedItemPosition());
            if (uhfH.keySave(keyType)) {
                Toast.makeText(SafetyManageActivity.this, "保存密钥 成功", Toast.LENGTH_SHORT).show();
            } else {
                Toast.makeText(SafetyManageActivity.this, "保存密钥 失败", Toast.LENGTH_SHORT).show();
            }
        }
    }
    
    class BtSetAdfClickListener implements OnClickListener {
        
        @Override
        public void onClick(View arg0) {
            UhfH.ApplicationDataFile adf = UhfH.ApplicationDataFile.ValueOf(SpinnerAdf.getSelectedItemPosition());
            boolean isSuccess = uhfH.setADF(adf);
            if (isSuccess) {
                Toast.makeText(SafetyManageActivity.this, "设置 ADF 成功", Toast.LENGTH_SHORT).show();
            } else {
                Toast.makeText(SafetyManageActivity.this, "设置 ADF 失败", Toast.LENGTH_SHORT).show();
            }
        }
    }
    
    class BtGetAdfClickListener implements OnClickListener {
        
        @Override
        public void onClick(View arg0) {
            
            UhfH.ApplicationDataFile adf = uhfH.getADF();
            if (adf != null) {
                SpinnerAdf.setSelection(adf.getValue());
                Toast.makeText(SafetyManageActivity.this, "获取 ADF 成功", Toast.LENGTH_SHORT).show();
            } else {
                Toast.makeText(SafetyManageActivity.this, "获取 ADF 失败", Toast.LENGTH_SHORT).show();
            }
        }
    }
    
    private void GenerateRandom(byte[] random) {
        Random rand = new Random();
        rand.nextBytes(random);
    }
}
